/*

gcc main-0.1.1.c -o main-0.1.1.exe `sdl-config --cflags --libs` -lpng -lm -g

c:/devkitGP2X/bin/arm-linux-gcc main-0.1.1.c -o main.gpe -static -DGP2X -I"C:/devkitGP2X/include" -L"C:/devkitGP2X/lib" -lSDL -lpthread -lpng -lz -lm -msoft-float -O3 && cp main.gpe i:/ptennis.gpe

*/

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <float.h>
#include <math.h>
#include <time.h>
#include <png.h>
#ifdef GP2X
	#include <SDL/SDL.h>
	#include <SDL/SDL_audio.h>
#else
	#include <SDL.h>
	#include <SDL_audio.h>
#endif

#ifdef GP2X
	#include <unistd.h>
	#include <fcntl.h>
	#include <sys/mman.h>

	#define GPB_N		gpio[0]
	#define GPB_NW		gpio[1]
	#define GPB_W		gpio[2]
	#define GPB_SW		gpio[3]
	#define GPB_S		gpio[4]
	#define GPB_SE		gpio[5]
	#define GPB_E		gpio[6]
	#define GPB_NE		gpio[7]
	#define GPB_START	gpio[8]
	#define GPB_SELECT	gpio[9]
	#define GPB_L		gpio[10]
	#define GPB_R		gpio[11]
	#define GPB_A		gpio[12]
	#define GPB_B		gpio[13]
	#define GPB_X		gpio[14]
	#define GPB_Y		gpio[15]
	#define GPB_VOLP	gpio[16]
	#define GPB_VOLM	gpio[17]
	#define GPB_CLICK	gpio[18]

	#define SYS_CLK_FREQ	7372800
	#define FPLLSETVREG	0x910

	int8_t gpio[19];
	uint16_t *memregs16;
	uint32_t *memregs32;
	int32_t devgpio;
	int32_t devmem;
	int32_t devfb;
#endif

#define pi 3.1415926535897932

#define SOUND_SAMPLERATE	44100
#define SOUND_BUFFERLEN		512

#define GP2X_SPEED	80	// GP2X CPU speed in MHz

#define FRAC 		6	//fractional format (here 6 for the 10.6 format)
#define SHADOW_INTENS 	170	//Shadow intensity in 0.8 format (0 = black shadows, 256 = no shadows)
#define REAL_SHADOW	0	// 0 : shadow right under the ball, 1 : shadow realistically projected to the side
#define SCREEN_W 	150
#define SCREEN_H 	120
#define SCREEN_SCALE 	4
#define	CENTER_X 	((int32_t) (74.5f * (float) (1<<FRAC)))	//- 25 = 48
#define CENTER_Y 	((int32_t) ((32.0f + 50.5f) * (float) (1<<FRAC)))	//- 26 = 24
#define NET_HEIGHT	(10<<FRAC)		// Net height in dm (default 10)
#define SIDE_DISTANCE	0			// distance to the side of cubes in 0.8 format  (default 0, old default : 30)
#define MOV_DIST	16			// distance from the box at which the box will start to move (default 16)
#define MOV_RANGE	12			// random range from MOV_DIST upwards (i.e. the range is [16; 16+12])  (default 12)
#define JD_MIN 		30			// JD = jump distance. Adds up to the movement distance for when to start moving only (default 30)
#define JD_RANGE	32			// (default 32)
#define JUMP_ODDS	((int32_t) (256.0 * 1.0/6.0))		// odds in 0.8 format. 32 -> 32 / 256 -> 1 / 8 odds  TODO rubberband value depending on what the player does
#define SLOMO		1		// FIXME the time speed system (make it float based or something) mmh no actually just use fractions
#define FASTFWD		1

#define	BOX_MAX 	(13<<FRAC)	// default 8, 13, 21 also 15
#define	BOX_MIN 	(5<<FRAC)	// default 3, 5, 8   also 6
#define BOX_THICK	(BOX_MIN>>FRAC)		// default 3, 5, 8
#define COURT_WIDTH	(14<<FRAC)	// half width of the aimable court
#define BOX_SPD 	((int32_t) (60.0f * (float) (1<<FRAC)))	// speed of lateral walking on ground (60.f = 21.6 km/h)
#define BOX_JMPSPD 	((int32_t) (48.0f * (float) (1<<FRAC)))	// speed of jumping sequence (= mid-air lateral speed) (48.f = 17.28 km/h)
#define BOX_FLYSPD 	((int32_t) (13.0f * (float) (1<<FRAC)))	// wobble speed in mid-air
#define BOX_LANDSPD 	((int32_t) (48.0f * (float) (1<<FRAC)))	// speed of deformation when landing (which blocks lateral movement)
#define BOX_STOPSPD 	((int32_t) (16.0f * (float) (1<<FRAC)))	// unused
#define TGT_ANGLE_MIN	80		// minimum target angle when throwing a high ball as to not throw a too fast and too direct ball (default 80)
#define HIBALL_TGT_ALT	22		// target altitude for the high ball (formerly BOX_MAXJMP - ((BOX_MIN>>1)+1>>FRAC)) (default 22)

#define GROUND_BOUNCE	0.76f	// the 0.76 value matches to real-world experimental data
#define WALL_BOUNCE	0.4f
#define FAR_WALL_Z	(-60)
#define NEAR_WALL_Z	90

#define LOSS_BALL_AI_STOP	250	// the AI stops moving 250 ms after the ball was lost
#define BALL_LOSS_MAX_TIME	2500	// the game is reset after 2500 ms the ball is lost at most

/*
 * LUT gen based on LUT_ERR
   0.1	10.755	<- best precision achievable with the current settings
   0.2	 9.782
   0.5	 8.480
   1.0	 7.586
   10.0	 4.759
   100.0 3.206

*/

// The following defines require a LUT regeneration if modified
#define FAILURE_SPD	(-1)//(-100<<FRAC)	// speed to set if throw parameters are incorrect
#define BOX_D		44		// distance of the first side of boxes to the net in dm
#define BOX_MAXJMP	25		// max possible distance of the center of the box from the ground when jumping high
#define BALL_TV 	312.928f	// ball terminal velocity in dm/s (312.928f = 112.65 km/h)
#define GRAV 		98.1f		// was 97.4f, lower gravity to compensate the 0.7% off in the measurement of time
#define BALL_RADIUS	(1<<FRAC)

// LUT = 4 bytes * LUT_AC * LUT_HC * LUT_DC = 4 * 144 * 64 * 32 = 4 * 294912 = 1152 kB
#define LUT_MS		(20 * 66)					// 20 ms = 50 FPS
#define LUT_ERR		((int32_t) (0.13f * (float) (1<<FRAC)))		// Accepted height error (8 / 64)
#define LUT_DC		32						// Distances count, going from 88 to 104
#define LUT_DST_STEP	((int32_t) (0.5f * (float) (1<<FRAC)))		// <- This and ^ this are inter-dependant
#define LUT_MIN_DST	((BOX_D<<(1+FRAC)) - (BALL_RADIUS<<1))
#define LUT_MIN_SPD	(25<<FRAC)	// 9 km/h
#define LUT_MAX_SPD	(570<<FRAC)	// 205 km/h
#define LUT_START_SPD	(150<<FRAC)	// 54 km/h

#define LUT_ANG_FMT	8	// 0.8 format
#define LUT_LOW_ANG	((int32_t) (((float) -BOX_MAXJMP / (float) (BOX_D<<1)) * (float) (1<<LUT_ANG_FMT)))	// now : -22/88 = -0.25
#define LUT_HIG_ANG	((int32_t) (2.0f * (float) (1<<LUT_ANG_FMT)))
#define LUT_AC		((int32_t) (64.0f * (2.0f - ((float) -BOX_MAXJMP / (float) (BOX_D<<1)))))		// Angles count
#define LUT_ANG_STEP	((int32_t) ((1.0f/64.0f) * (float) (1<<LUT_ANG_FMT)))

#define LUT_MIN_HEI	(-32<<FRAC)
#define LUT_HEI_STEPF	1.0f
#define LUT_HEI_STEP	((int32_t) (LUT_HEI_STEPF * (float) (1<<FRAC)))
#define LUT_HC		  ((int32_t) (64.0f / LUT_HEI_STEPF))	// 64 / 1.0 = 64


typedef struct
{
	int16_t *snd;		// sound data
	int32_t len;		// sound length (in samples)
	int32_t pos;		// next sample to play, won't play if out of bounds, will schedule if negative
	int32_t vol;		// volume (0.16 format)
	int32_t vol_mod;	// global volume modifier in 8.8 format
} sound_t;

sound_t bounce_sound;
sound_t hit_sound;
sound_t fast_sound;

int32_t i=0, foobar=0;
int64_t somme=0;
int32_t max_h=0;
int32_t tgt_x;
int8_t hitting_box=0, prev_hb;	// previous hitting box
int8_t ball_jump=0;

int32_t center_x = CENTER_X;
int32_t center_y = CENTER_Y;

int32_t service = 0;	// 1 only if a box has just served the ball

int32_t *global_p;

typedef struct
{
	int32_t speed;
	int16_t height;
} lut_entry;

lut_entry ***lut;
int16_t **min_angle;	// array containing the minimal angle needed depending on the distance and target height

typedef struct
{
	int32_t x, y, z;	//coord in 10.6 format
} coord_t;

typedef struct
{
	uint32_t *RGBA;
	uint8_t *alpha;
	uint16_t width, height;
	int32_t x, y, z;	//coordinates are in 10.6 format
} object_t;

typedef struct
{
	uint32_t *RGBA;
	uint8_t *alpha;
	uint16_t width, height;
	int32_t x, y, z;	//coordinates are in 10.6 format
	coord_t speed, prev, col;
	int32_t col_ratio;
} ball_t;

typedef struct
{
	char name;	// box's code, either 1 or 2
	int32_t x1, x2, y1, y2, z1, z2; 	//box characteristics in 10.6 format
	uint16_t colorA, colorB, colorC;	//RGB565 colors for each side of the box
	int8_t left, right, jump, phys_jump;
	int8_t moving;	// 0 means not moving, 1 means left leg is moving, 2 means right leg is moving
	int8_t flying;	// 0 means on the ground, 1 means jumping, 2 means aborpted jump, 3 means flying, 4 means landing
	int32_t geo_shape, geo_speed;
	coord_t g, speed, size;	// g is the center of gravity of the box, speed is g's speed
	int8_t ai;	// Artificial Intelligence
	int8_t left_of_0, left_of_tgt_x;
	int32_t pre_jump_g_x, pre_jump_time;
	int8_t stop;
	int32_t randir, jump_distance, mov_distance;
} box_t;

typedef struct
{
	int32_t xmin, xmax;
} fill_array_entry_t;

typedef struct
{
	int32_t y_offset, y_end, size;	//offset of the first pixel from the top of the screen
	fill_array_entry_t line[SCREEN_H];
} fill_array_t;

fill_array_t fa;

/*inline int32_t speed(int32_t x, int32_t y, int32_t z)	// everything is in 10.6 format (unused)
{
	int32_t max=x, min_a=x, min_b;

	if (x<0)
		x = -x;
	if (y<0)
		y = -y;
	if (z<0)
		z = -z;

	if (y>max)
		max = y;
	else
		min_a = y;

	if (z>max)
	{
		min_b = max;
		max = z;
	}
	else
		min_b = z;

	return max + (min_a >> 2) + (min_b >> 2);
}*/

inline int32_t speed2(int32_t x, int32_t y, int32_t z)	// everything is in 10.6 format
{
	x = x * x >> (FRAC<<1);	// now in 20.0
	y = y * y >> (FRAC<<1);
	z = z * z >> (FRAC<<1);

	return x + y + z;	// now in 22.0
}

int32_t sqrtq(int32_t x)
{
	uint32_t root = 0, xhi = 0, xlo, div;
	int32_t count = 15 + (FRAC>>1); // 15 + (number of fractional bits/2)??
 
	if (x < 0)
		x = -x; // naughty

	xlo = x;
 
	do
	{
		xhi = (xhi << 2) | (xlo >> 30);
		xlo <<= 2;
		root <<= 1;
		div = (root << 1) + 1;
 
		if (xhi >= div)
		{
			xhi -= div;
			root++;
		}
	} while (count-- != 0);
 
	return (int32_t) root;
}

inline int32_t sign(int32_t x)
{
	if (x == 0)
		return 0;
	else
		if (x > 0)
			return 1;
		else
			return -1;
}

int32_t distance(int32_t x1, int32_t y1, int32_t x2, int32_t y2)	// all is in format 10.6
{
	int32_t a, x, y;

	x = x2 - x1;
	y = y2 - y1;

	a = (x*x >> FRAC) + (y*y >> FRAC);	// a is distance

	return sqrtq(a);
}

#ifdef GP2X
void nsleep(uint32_t s, uint32_t ns)	//simplifies calls to nanosleep
{
	struct timespec tspec;

	tspec.tv_sec = (time_t) s;
	tspec.tv_nsec = ns;

	nanosleep(&tspec, NULL);
}

void io_parse(uint32_t io_word, uint32_t prev_io)
{
	uint32_t diff = prev_io ^ io_word;
	uint32_t i;

	for (i=0; i<19; i++)
		if ((diff>>i) & 1 == 1)
			gpio[i]	= (io_word>>i) & 1;

}
#endif

#ifdef GP2X
__inline uint8_t upperpower(int32_t n)
#else
uint8_t upperpower(int32_t n)
#endif
{
	uint8_t p=0;

	while (n!=0)
	{
		n>>=1;
		p++;
	}

	return p;
}

uint32_t rand_u32()
{
	uint32_t x;

	x = ((rand()%256)<<24) | ((rand()%256)<<16) | ((rand()%256)<<8) | (rand()%256);

	return x;
}

void blit_center(uint16_t *screen, uint32_t *sprite, uint16_t width, uint16_t height, int16_t center_x, int16_t center_y, char mode)
{	//sprite is in the RGBAX56588 format
	int32_t iySCREEN_W, start_ySCREEN_W, iy_width;
	int16_t ix, iy, pos_x, pos_y, offset_x, offset_y, start_x, start_y, stop_x, stop_y;
	uint8_t alpha_pixel;

	/*List of blending modes
	 * 0 = replace value, no alpha blending
	 * 1 = replace value, optimized alpha blending
	 */

	pos_x = center_x - (width>>1);
	pos_y = center_y - (height>>1);

	if (pos_x < 0)
	{
		offset_x = 0;
		start_x = -pos_x;
	}
	else
	{
		offset_x = pos_x;
		start_x = 0;
	}

	if (pos_y < 0)
	{
		offset_y = 0;
		start_y = -pos_y;
	}
	else
	{
		offset_y = pos_y;
		start_y = 0;
	}

	if (pos_x + width >= SCREEN_W)
		stop_x = SCREEN_W - pos_x;
	else
		stop_x = width;

	if (pos_y + height >= SCREEN_H)
		stop_y = SCREEN_H - pos_y;
	else
		stop_y = height;

	start_ySCREEN_W = start_y * SCREEN_W;

	/*
	for (iy=start_y; iy<stop_y; iy++)
	{
		iySCREEN_W = (iy + offset_y) * SCREEN_W;
		iy_width = iy * width;
		for (ix=start_x; ix<stop_x; ix++)
		{
			if (mode==0)
				screen[iySCREEN_W + ix + offset_x - start_ySCREEN_W - start_x] = sprite[iy_width + ix] >> 16;
			if (mode==1)
			{
				alpha_pixel = ((uint8_t *)sprite)[((iy_width+ix)<<2) + 2];
				if (alpha_pixel==255)
					screen[iySCREEN_W + ix + offset_x - start_ySCREEN_W - start_x] = sprite[iy_width + ix] >> 16;
				if (alpha_pixel!=255 && alpha_pixel!=0)
					screen[iySCREEN_W + ix + offset_x - start_ySCREEN_W - start_x] =
(((((sprite[iy_width+ix]>>16)&0xF800) * alpha_pixel) >> 8) + (((screen[iySCREEN_W + ix + offset_x - start_ySCREEN_W - start_x]&0xF800) * (255-alpha_pixel)) >> 8)) |
(((((sprite[iy_width+ix]>>16)&0x07E0) * alpha_pixel) >> 8) + (((screen[iySCREEN_W + ix + offset_x - start_ySCREEN_W - start_x]&0x07E0) * (255-alpha_pixel)) >> 8)) |
(((((sprite[iy_width+ix]>>16)&0x001F) * alpha_pixel) >> 8) + (((screen[iySCREEN_W + ix + offset_x - start_ySCREEN_W - start_x]&0x001F) * (255-alpha_pixel)) >> 8));
			}
		}
	}*/

	if (mode==0)
		for (iy=start_y; iy<stop_y; iy++)
		{
			iySCREEN_W = (iy + offset_y) * SCREEN_W;
			iy_width = iy * width;
			for (ix=start_x; ix<stop_x; ix++)
				screen[iySCREEN_W + ix + offset_x - start_ySCREEN_W - start_x] = sprite[iy_width + ix] >> 16;
		}

	if (mode==1)
		for (iy=start_y; iy<stop_y; iy++)
		{
			iySCREEN_W = (iy + offset_y) * SCREEN_W;
			iy_width = iy * width;
			for (ix=start_x; ix<stop_x; ix++)
			{
				alpha_pixel = ((uint8_t *)sprite)[((iy_width+ix)<<2) + 1];
				if (alpha_pixel==255)
					screen[iySCREEN_W + ix + offset_x - start_ySCREEN_W - start_x] = sprite[iy_width + ix] >> 16;
				if (alpha_pixel!=255 && alpha_pixel!=0)
					screen[iySCREEN_W + ix + offset_x - start_ySCREEN_W - start_x] =
((((((sprite[iy_width+ix]>>16)&0xF800) * alpha_pixel) >> 9) + (((screen[iySCREEN_W + ix + offset_x - start_ySCREEN_W - start_x]&0xF800) * (255-alpha_pixel)) >> 9))&0xF800) |
((((((sprite[iy_width+ix]>>16)&0x07E0) * alpha_pixel) >> 9) + (((screen[iySCREEN_W + ix + offset_x - start_ySCREEN_W - start_x]&0x07E0) * (255-alpha_pixel)) >> 9))&0x07E0) |
((((((sprite[iy_width+ix]>>16)&0x001F) * alpha_pixel) >> 9) + (((screen[iySCREEN_W + ix + offset_x - start_ySCREEN_W - start_x]&0x001F) * (255-alpha_pixel)) >> 9))&0x001F);
			}
		}
}

void text_blit(uint16_t *screen, char *string, uint8_t *font, uint8_t char_w, uint8_t char_h, uint16_t x, uint16_t y)
{
	uint32_t i=0;
	uint16_t ix, iy;
	uint8_t font_pixel;
	static char init=1;
	static uint32_t limit;

	if (init)			//leave it as such as long as always the same font width is used
		limit = SCREEN_W / char_w;
	init = 0;

	while (string[i]!=0 && i<limit)
	{
		if (string[i]<' ')
			string[i] = ' ';

		for (iy=y; iy<y+char_h; iy++)
		{
			for (ix=x; ix<x+char_w; ix++)
			{
				font_pixel = font[(((string[i]>>4)-2)*char_h+iy-y)*96 + (string[i] & 0xf)*char_w + ix-x] & 248;
				if (font_pixel!=0)
					screen[iy*SCREEN_W + ix + char_w*i] = 0;
			}
		}
		i++;
	}
}

void add_alpha(uint32_t *RGBAX, uint8_t *alpha, uint16_t width, uint16_t height, uint16_t power, uint8_t mode)
{	//RGBAX is in the RGBAX56588 format
	uint16_t ix, iy;

	/*List of modes
	 * 0 = Power index mode
	 * 1 = Normal index mode
	 */

	if (mode==0)
		for (iy=0; iy<height; iy++)
			for (ix=0; ix<width; ix++)
				RGBAX[(iy*((1<<power)))+ix] |= alpha[iy*width + ix] << 8;

	if (mode==1)
		for (iy=0; iy<height; iy++)
			for (ix=0; ix<width; ix++)
				RGBAX[(iy*(width))+ix] |= alpha[iy*width + ix] << 8;
}

#ifndef GP2X
void surf_array_o(SDL_Surface *surface, uint16_t *image, uint16_t width, uint16_t height)	//puts pixels from an array to a surface (optimized)
//16 bit version
{
	int32_t iy_pitch=0, index=0;
	uint16_t ix, iy, pitch_value=surface->pitch;
	uint8_t *p;
	void *pixels_start = surface->pixels;

	//RRRRRggggggBBBBB
	//		RRRRRrrr
	//		GGGGGGgg
	//		BBBBBbbb
 
	for (iy=0; iy<height; iy++)
	{
		for (ix=0; ix<width; ix++)
		{
			   p = (uint8_t *) pixels_start + iy_pitch + ix*4;		//p == address to the pixel
			   p[2] = (image[index]&0xF800)>>8;
			   p[1] = (image[index]&0x07E0)>>3;
			   p[0] = (image[index]&0x001F)<<3;
			   ++index;
		}
		iy_pitch+=pitch_value;
	}

}
#endif

void scale(uint16_t *i, uint16_t *o, uint16_t height, uint16_t width, int8_t factor)
{
	uint16_t iy, ix, jy, jx;

	for (iy=0; iy<height; iy++)
		for (ix=0; ix<width; ix++)
			for (jy=0; jy<factor; jy++)
				for (jx=0; jx<factor; jx++)
					o[(iy*factor + jy)*(SCREEN_W*factor) + ix*factor + jx] = i[iy * SCREEN_W + ix];
}

//"stolen" from XBubble's source code and slightly modified to my liking
uint8_t *load_png_file(const char *file, uint16_t *width, uint16_t *height, uint8_t *i_power, uint8_t mode)
{
	/* Modes :
	 * 0 : RGB888 w/ power
	 * 1 : RGB888
	 * 2 : 8-bit grey
	 */

	FILE *fd;
	uint8_t *data, *image, header[8], ic;
	int32_t  bit_depth, color_type, i, iy, ix;
	png_uint_32  png_width, png_height, rowbytes;
	png_structp png_ptr;
	png_infop info_ptr;
	png_bytep *row_pointers;

	fd = fopen( file, "rb" );
	if ( fd == NULL )
	{
		perror(file);
		return NULL;
	}
	// ensure that we opened a PNG file
	fread( header, 1, 8, fd );
	if ( ! png_check_sig( header, 8 ) )
	{
		fclose(fd);
		printf("File %s does not have a valid PNG signature.\n", file);
		return NULL;
	}
	// create PNG structs
	png_ptr = png_create_read_struct( PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
	if ( ! png_ptr )
	{
		fclose(fd);
		return NULL;
	}
	info_ptr = png_create_info_struct(png_ptr);
	if ( ! info_ptr )
	{
		png_destroy_read_struct( &png_ptr, (png_infopp)NULL, (png_infopp)NULL);
		fclose(fd);
		return NULL;
	}
	//libpng does a longjmp here when it encounters an error
	if ( setjmp( png_ptr->jmpbuf ) )
	{
		png_destroy_read_struct( &png_ptr, &info_ptr, NULL);
		fclose(fd);
		return NULL;
	}
	png_init_io( png_ptr, fd );
	png_set_sig_bytes( png_ptr, 8);
	png_read_info( png_ptr, info_ptr);
	png_get_IHDR( png_ptr, info_ptr, &png_width, &png_height, &bit_depth,
				  &color_type, NULL, NULL, NULL);
	*width = (int16_t) png_width;
	*height = (int16_t) png_height;

/*********************SHOULDN'T BE NEEDED****************************/
	//convert image format to RGB(A)
	if ( color_type == PNG_COLOR_TYPE_PALETTE )
		png_set_expand(png_ptr);

	if (( color_type == PNG_COLOR_TYPE_GRAY ) || ( color_type == PNG_COLOR_TYPE_GRAY_ALPHA ) && mode!=3)
		png_set_gray_to_rgb(png_ptr);

	// we only handle 8 bits per channel
	if (bit_depth == 16 && mode!=3)
		png_set_strip_16(png_ptr);
	// insert filling bytes if necessary
	png_set_filler(png_ptr, 0xff, PNG_FILLER_AFTER);

	// update info data
	png_read_update_info( png_ptr, info_ptr);
/********************************************************************/

	// allocate space for data and row pointers
	rowbytes = png_get_rowbytes( png_ptr, info_ptr);
	data = malloc (rowbytes * (*height));
		if (data == NULL)
				fprintf(stderr, "malloc returned null on data\n");
	row_pointers = malloc ((*height) * sizeof(png_bytep));
		if (row_pointers == NULL)
				fprintf(stderr, "malloc returned null on row_pointers\n");
	if (( data == NULL ) || ( row_pointers == NULL ))
	{
		png_destroy_read_struct( &png_ptr, &info_ptr, NULL);
		free(data);
		free(row_pointers);
		return NULL;
	}
	// initialize row pointers
	for ( i = 0;  i < *height; i++ )
		row_pointers[i] = data + i * rowbytes;
	// read all image data
	png_read_image( png_ptr, row_pointers );
	png_read_end( png_ptr, NULL);
	// cleanup
	free(row_pointers);
	png_destroy_read_struct( &png_ptr, &info_ptr, NULL);
	fclose(fd);
	// return an array of RGB(A) values
	//return data;

	*i_power=upperpower(*width);

	if (mode==0)	//RGBX8888 w/ power
	{
		image = malloc ((1<<*i_power) * (*height) * 4 * sizeof(uint8_t));
		if (image == NULL)
			fprintf(stderr, "malloc returned null on image\n");
		for (iy=0; iy<*height; iy++)
			for (ix=0; ix<(1<<*i_power); ix++)
				for (ic=0; ic<3; ic++)
					image[(((iy*((1<<*i_power)))+ix)<<2)+ic] = data[(iy * (*width) + ix) * 4 + ic];
	}

	if (mode==1)	//RGBX8888
	{
		image = malloc ((*width) * (*height) * 4 * sizeof(uint8_t));
	 	if (image == NULL)
		 	fprintf(stderr, "malloc returned null on image\n");
		for (iy=0; iy<*height; iy++)
			for (ix=0; ix<*width; ix++)
				for (ic=0; ic<3; ic++)
					image[(((iy*(*width))+ix)<<2)+ic] = data[(iy * (*width) + ix) * 4 + ic];
	}

	if (mode==2)	//8-bit grey
	{
		image = malloc ((*width) * (*height) * sizeof(uint8_t));
		if (image == NULL)
			fprintf(stderr, "malloc returned null on image\n");
		for (iy=0; iy<*height; iy++)
			for (ix=0; ix<*width; ix++)
				image[iy * (*width) + ix] =  data[(iy * (*width) + ix) * 4];
	}

	if (mode==3)	//16-bit grey TODO test
	{
		image = malloc ((*width) * (*height) * sizeof(uint16_t));
		if (image == NULL)
			fprintf(stderr, "malloc returned null on image\n");
		for (iy=0; iy<*height; iy++)
			for (ix=0; ix<*width; ix++)
				((uint16_t *)image)[iy * (*width) + ix] =  ((uint16_t *)data)[(iy * (*width) + ix)];
	}

	if (mode==4)	//RGBXX56588 w/ power
	{	//data is in XBGR8888 format
		//XXXXXXXXBBBBBbbbGGGGGGggRRRRRrrr	<< XBGR8888
		//RRRRRGGGGGGBBBBBxxxxxxxxxxxxxxxx	<< RGBXX56588
		//
		//0xFF775533
		image = malloc ((1<<*i_power) * (*height) * sizeof(uint32_t));
		if (image == NULL)
			fprintf(stderr, "malloc returned null on image\n");
		for (iy=0; iy<*height; iy++)
			for (ix=0; ix<*width; ix++)
				((uint32_t *) image)[(iy*((1<<*i_power)))+ix] =
					((((uint32_t *) data)[(iy * (*width) + ix)]&0x00F80000) >> 3) |
					((((uint32_t *) data)[(iy * (*width) + ix)]&0x0000FC00) << 11) |
					((((uint32_t *) data)[(iy * (*width) + ix)]&0x000000F8) << 24);
	}

	if (mode==5)	//RGBXX56588
	{
		image = malloc ((*width) * (*height) * sizeof(uint32_t));
		if (image == NULL)
			fprintf(stderr, "malloc returned null on image\n");
		for (iy=0; iy<*height; iy++)
			for (ix=0; ix<*width; ix++)
				((uint32_t *) image)[iy*(*width)+ix] =
					((((uint32_t *) data)[(iy * (*width) + ix)]&0x00F80000) >> 3) |
					((((uint32_t *) data)[(iy * (*width) + ix)]&0x0000FC00) << 11) |
					((((uint32_t *) data)[(iy * (*width) + ix)]&0x000000F8) << 24);
	}

	free(data);
	return image;
}

void coord_conv(int32_t x, int32_t y, int32_t z, int32_t *X, int32_t *Y)
{
	*X = -x + z + CENTER_X >> FRAC;
	*Y = (x>>1) - y + (z>>1) + CENTER_Y >> FRAC;
}

int32_t coord_convX(int32_t x, int32_t y, int32_t z)
{
	return -x + z + CENTER_X >> FRAC;
}

int32_t coord_convY(int32_t x, int32_t y, int32_t z)
{
	return (x>>1) - y + (z>>1) + CENTER_Y >> FRAC;
}

int32_t coord_convZ(int32_t x, int32_t y, int32_t z)	//screen Z-axis goes towards you, thus, the larger the value the closer to you
{
	return (((x>>1) + (z>>1))>>FRAC);
}

void draw_circle(uint16_t *screen16, float size)
{
	int32_t i, x, y;
	float a=1024.0f;

	for (i=0; i<(int32_t) a; i++)
	{
		coord_conv(((int32_t) (cosf((float) i*pi*2.0f/a) * size * (16<<FRAC))), 0, ((int32_t) (sinf((float) i*pi*2.0f/a) * size * (16<<FRAC))), &x, &y);
		screen16[y * SCREEN_W + x] = 0xF810;
		//printf("x : %d, y : %d  cos : %.3f\n", x, y, (cosf((float) i*pi*2.0f/a) * (16<<FRAC)));
	}

}

/*void draw_line(uint16_t *screen16, int32_t x1, int32_t y1, int32_t x2, int32_t y2, uint16_t color)
{
	int32_t min, max;

	int32_t i;
	int32_t xd, yd;
	int32_t dir_x, dir_y;
	int32_t error_x, error_y;
	int32_t detdiff;
	int32_t count;

	if (x1<0 && x2<0)		return;
	if (y1<0 && y2<0)		return;
	if (x1>SCREEN_W && x2>SCREEN_W)	return;
	if (y1>SCREEN_H && y2>SCREEN_H)	return;

	//********horizontal line********
	if (x1 == x2 && (x1>=0 && x1<SCREEN_W))		// if horizontal and on-screen
	{
		if (y1 < y2)
		{
			min = y1;
			max = y2;
		}
		else
		{
			min = y2;
			max = y1;
		}

		if (min<0)		// if it starts out of bounds
			min = 0;

		if (max>=SCREEN_H)	// if it ends out of bounds
			max = SCREEN_H-1;

		for (i=min; i<max; i++)	
				screen16[i * SCREEN_W + x1] = color;

		return;
	}
	//--------horizontal line--------


	//********vertical line********
	if (y1 == y2 && (y1>=0 && y1<SCREEN_H))		// if vertical and on-screen
	{
		if (x1 < x2)
		{
			min = x1;
			max = x2;
		}
		else
		{
			min = x2;
			max = x1;
		}

		if (min<0)		// if it starts out of bounds
			min = 0;

		if (max>=SCREEN_W)	// if it ends out of bounds
			max = SCREEN_W-1;

		for (i=min; i<max; i++)	
				screen16[y1 * SCREEN_W + i] = color;

		return;
	}
	//--------vertical line--------


	dir_y = 1;
	yd = y2-y1;
	if (yd < 0)
	{
		dir_y = -1;
		yd = -yd;
	}

	dir_x = 1;
	xd = x2 - x1;
	if (xd < 0)
	{
		dir_x = -1;
		xd = -xd;
	}



	if (y1>=0 && y1<SCREEN_H && x1>=0 && x1<SCREEN_W)
		screen16[y1 * SCREEN_W + x1] = color;

	if (yd > xd)
	{
		count=yd;
		detdiff=yd;
		error_x=yd >> 1;
		error_y=0;
	}
	else
	{
		count=xd;
		detdiff=xd;
		error_x=0;
		error_y=xd >> 1;
	}

	do
	{
		error_y=(error_y+yd);
		if (error_y>=detdiff)
		{
			error_y-=detdiff;
			y1 += dir_y;
		}

		error_x=(error_x+xd);
		if (error_x>=detdiff)		// very slow
		{
			error_x-=detdiff;	// very slow
			x1 += dir_x;		// very slow
		}

		if (y1>=0 && y1<SCREEN_H && x1>=0 && x1<SCREEN_W)	// very slow
			screen16[y1 * SCREEN_W + x1] = color;

		count--;
	}					// very slow
	while (count>0);

	if (y1>=0 && y1<SCREEN_H && x1>=0 && x1<SCREEN_W)
		screen16[y1 * SCREEN_W + x1] = color;
}*/

void draw_line(uint16_t *screen16, int32_t x1, int32_t y1, int32_t x2, int32_t y2, uint16_t color)
{
	int32_t deltaRow, deltaCol;
	int32_t stepRow, stepCol;
	int32_t errorRow, errorCol;
	int32_t detdelta;
	int32_t count;

	int32_t rr1, rr2, cc1, cc2;

	rr1 = x1;
	rr2 = x2;
	cc1 = y1;
	cc2 = y2;

	// horizontal line
	if (rr1 == rr2)
	{
		if (cc1 > cc2)
			stepCol = -1;
		else
			stepCol = 1;

		for (; cc1 != cc2; cc1 += stepCol)
		{
			if (cc1>=0 && cc1<SCREEN_H && rr1>=0 && rr1<SCREEN_W)
				screen16[cc1 * SCREEN_W + rr1] = color;
		}

		if (cc1>=0 && cc1<SCREEN_H && rr1>=0 && rr1<SCREEN_W)
			screen16[cc1 * SCREEN_W + rr1] = color;
		return;
	}


	// vertical line
	if (cc1 == cc2)
	{
		if (rr1 > rr2)
			stepRow = -1;
		else
			stepRow = 1;

		for (; rr1 != rr2; rr1 += stepRow)
		{
			if (cc1>=0 && cc1<SCREEN_H && rr1>=0 && rr1<SCREEN_W)
				screen16[cc1 * SCREEN_W + rr1] = color;
		}

		if (cc1>=0 && cc1<SCREEN_H && rr1>=0 && rr1<SCREEN_W)
			screen16[cc1 * SCREEN_W + rr1] = color;
		return;
	}


	if (cc1>=0 && cc1<SCREEN_H && rr1>=0 && rr1<SCREEN_W)
		screen16[cc1 * SCREEN_W + rr1] = color;

	stepCol = 1;
	deltaCol = cc2-cc1;
	if (deltaCol < 0)
	{
		stepCol = -1;
		deltaCol = -deltaCol;
	}

	stepRow = 1;
	deltaRow = rr2 - rr1;
	if (deltaRow < 0)
	{
		stepRow = -1;
		deltaRow = -deltaRow;
	}

	if (deltaCol > deltaRow)
	{
		count=deltaCol;
		detdelta=deltaCol;
		errorRow=deltaCol >> 1;
		errorCol=0;
	}
	else
	{
		count=deltaRow;
		detdelta=deltaRow;
		errorRow=0;
		errorCol=deltaRow >> 1;
	}

	do
	{
		errorCol=(errorCol+deltaCol);
		if (errorCol>=detdelta)
		{
			errorCol-=detdelta;
			cc1 += stepCol;
		}

		errorRow=(errorRow+deltaRow);
		if (errorRow>=detdelta)
		{
			errorRow-=detdelta;
			rr1 += stepRow;
		}

		if (cc1>=0 && cc1<SCREEN_H && rr1>=0 && rr1<SCREEN_W)
			screen16[cc1 * SCREEN_W + rr1] = color;

		count--;
	}
	while ((int32_t)count>0);

	if (cc1>=0 && cc1<SCREEN_H && rr1>=0 && rr1<SCREEN_W)
		screen16[cc1 * SCREEN_W + rr1] = color;

}


/*inline void fill_line_array_entry(int32_t x1, int32_t y1)
{
	if (x1 < 0)
		x1 = -1;
	if (x1 >= SCREEN_W)
		x1 = SCREEN_W;

	if (x1 > fa.line[y1].xmax && y1>=0 && y1<SCREEN_H)
		fa.line[y1].xmax = x1;
	if (x1 < fa.line[y1].xmin && y1>=0 && y1<SCREEN_H)
		fa.line[y1].xmin = x1;
}

void draw_line_array(int32_t x1, int32_t y1, int32_t x2, int32_t y2)	//instead of drawing a line, fills an array used for polygon filling
{
	int32_t min, max;

	int32_t xd, yd;
	int32_t dir_x, dir_y;
	int32_t error_x, error_y;
	int32_t detdiff;
	int32_t count;

	if (x1<0 && x2<0)		return;
	if (y1<0 && y2<0)		return;
	if (x1>SCREEN_W && x2>SCREEN_W)	return;
	if (y1>SCREEN_H && y2>SCREEN_H)	return;

	//********horizontal line********
	if (x1 == x2 && (x1>=0 && x1<SCREEN_W))		// if horizontal and on-screen
	{
		if (y1 < y2)
		{
			min = y1;
			max = y2;
		}
		else
		{
			min = y2;
			max = y1;
		}

		if (min<0)		// if it starts out of bounds
			min = 0;

		if (max>=SCREEN_H)	// if it ends out of bounds
			max = SCREEN_H-1;

		for (y1=min; y1<max; y1++)
		{
			fill_line_array_entry(x1, y1);
		}

		return;
	}
	//--------horizontal line--------


	//********vertical line********
	if (y1 == y2 && (y1>=0 && y1<SCREEN_H))		// if vertical and on-screen
	{
		if (x1 < x2)
		{
			min = x1;
			max = x2;
		}
		else
		{
			min = x2;
			max = x1;
		}

		if (min<0)		// if it starts out of bounds
			min = 0;

		if (max>=SCREEN_W)	// if it ends out of bounds
			max = SCREEN_W-1;

		for (y1=min; y1<max; y1++)	
		{
			fill_line_array_entry(x1, y1);
		}

		return;
	}
	//--------vertical line--------


	dir_y = 1;
	yd = y2-y1;
	if (yd < 0)
	{
		dir_y = -1;
		yd = -yd;
	}

	dir_x = 1;
	xd = x2 - x1;
	if (xd < 0)
	{
		dir_x = -1;
		xd = -xd;
	}



	if (y1>=0 && y1<SCREEN_H)
	{
		fill_line_array_entry(x1, y1);
	}

	if (yd > xd)
	{
		count=yd;
		detdiff=yd;
		error_x=yd >> 1;
		error_y=0;
	}
	else
	{
		count=xd;
		detdiff=xd;
		error_x=0;
		error_y=xd >> 1;
	}

	do
	{
		error_y=(error_y+yd);
		if (error_y>=detdiff)
		{
			error_y-=detdiff;
			y1 += dir_y;
		}

		error_x=(error_x+xd);
		if (error_x>=detdiff)		// very slow
		{
			error_x-=detdiff;	// very slow
			x1 += dir_x;		// very slow
		}

		if (y1>=0 && y1<SCREEN_H)	// very slow
		{
			fill_line_array_entry(x1, y1);
		}

		count--;
	}					// very slow
	while (count>0);

	if (y1>=0 && y1<SCREEN_H)
	{
		fill_line_array_entry(x1, y1);
	}
}*/

void draw_line_array(int32_t x1, int32_t y1, int32_t x2, int32_t y2)	//instead of drawing a line, fills an array used for polygon filling
{
	int32_t deltaRow, deltaCol;
	int32_t stepRow, stepCol;
	int32_t errorRow, errorCol;
	int32_t detdelta;
	int32_t count;

	// horizontal line
	if (x1 == x2)
	{
		if (y1 > y2)
			stepCol = -1;
		else
			stepCol = 1;

		for (; y1 != y2; y1 += stepCol)
		{
			if (x1 > fa.line[y1].xmax && y1>=0 && y1<SCREEN_H)
				fa.line[y1].xmax = x1;
			if (x1 < fa.line[y1].xmin && y1>=0 && y1<SCREEN_H)
				fa.line[y1].xmin = x1;
		}

		if (x1 > fa.line[y1].xmax && y1>=0 && y1<SCREEN_H)
			fa.line[y1].xmax = x1;
		if (x1 < fa.line[y1].xmin && y1>=0 && y1<SCREEN_H)
			fa.line[y1].xmin = x1;
		return;
	}


	// vertical line
	if (y1 == y2)
	{
		if (x1 > x2)
			stepRow = -1;
		else
			stepRow = 1;

		for (; x1 != x2; x1 += stepRow)
		{
			if (x1 > fa.line[y1].xmax && y1>=0 && y1<SCREEN_H)
				fa.line[y1].xmax = x1;
			if (x1 < fa.line[y1].xmin && y1>=0 && y1<SCREEN_H)
				fa.line[y1].xmin = x1;
		}

		if (x1 > fa.line[y1].xmax)
			fa.line[y1].xmax = x1;
		if (x1 < fa.line[y1].xmin)
			fa.line[y1].xmin = x1;
		return;
	}


	if (x1 > fa.line[y1].xmax && y1>=0 && y1<SCREEN_H)
		fa.line[y1].xmax = x1;
	if (x1 < fa.line[y1].xmin && y1>=0 && y1<SCREEN_H)
		fa.line[y1].xmin = x1;

	stepCol = 1;
	deltaCol = y2-y1;
	if (deltaCol < 0)
	{
		stepCol = -1;
		deltaCol = -deltaCol;
	}

	stepRow = 1;
	deltaRow = x2 - x1;
	if (deltaRow < 0)
	{
		stepRow = -1;
		deltaRow = -deltaRow;
	}

	if (deltaCol > deltaRow)
	{
		count=deltaCol;
		detdelta=deltaCol;
		errorRow=deltaCol >> 1;
		errorCol=0;
	}
	else
	{
		count=deltaRow;
		detdelta=deltaRow;
		errorRow=0;
		errorCol=deltaRow >> 1;
	}

	do
	{
		errorCol=(errorCol+deltaCol);
		if (errorCol>=detdelta)
		{
			errorCol-=detdelta;
			y1 += stepCol;
		}

		errorRow=(errorRow+deltaRow);
		if (errorRow>=detdelta)
		{
			errorRow-=detdelta;
			x1 += stepRow;
		}

		if (x1 > fa.line[y1].xmax && y1>=0 && y1<SCREEN_H)
			fa.line[y1].xmax = x1;
		if (x1 < fa.line[y1].xmin && y1>=0 && y1<SCREEN_H)
			fa.line[y1].xmin = x1;

		count--;
	}
	while ((int32_t)count>0);

	if (x1 > fa.line[y1].xmax && y1>=0 && y1<SCREEN_H)
		fa.line[y1].xmax = x1;
	if (x1 < fa.line[y1].xmin && y1>=0 && y1<SCREEN_H)
		fa.line[y1].xmin = x1;

}

void fill_polygon(uint16_t *screen16, int32_t x1, int32_t y1, int32_t x2, int32_t y2, int32_t x3, int32_t y3, int32_t x4, int32_t y4, uint16_t color)
{
	int32_t iy, ix;

	fa.y_offset = y1;	// looking for the top point
	if (y2 < fa.y_offset)
		fa.y_offset = y2;
	if (y3 < fa.y_offset)
		fa.y_offset = y3;
	if (y4 < fa.y_offset)
		fa.y_offset = y4;

	fa.y_end = y1;	// looking for the lowest point
	if (y2 > fa.y_end)
		fa.y_end = y2;
	if (y3 > fa.y_end)
		fa.y_end = y3;
	if (y4 > fa.y_end)
		fa.y_end = y4;

	fa.y_end++;

	if (fa.y_end > SCREEN_H)
		fa.y_end = SCREEN_H;

//printf("%d   %d\n", fa.y_offset, fa.y_end);

	for (iy=fa.y_offset; iy<fa.y_end; iy++)
	{
		if (iy>=0)
		{
			fa.line[iy].xmin = SCREEN_W;
			fa.line[iy].xmax = -1;
		}
	}

	draw_line_array(x1, y1, x2, y2);
	draw_line_array(x3, y3, x2, y2);
	draw_line_array(x3, y3, x4, y4);
	draw_line_array(x1, y1, x4, y4);

	for (iy=fa.y_offset; iy<fa.y_end; iy++)
	{
		if (iy>=0 && iy<SCREEN_H)
			for (ix=fa.line[iy].xmin; ix<=fa.line[iy].xmax; ix++)
			{
				if (ix>=0 && ix<SCREEN_W)
				{
					screen16[iy * SCREEN_W + ix] = color;
				}
			}
	}
}

void draw_box(uint16_t *screen16, box_t box)
{
	int32_t x1, x2, x3, x4;
	int32_t iy, y1, y2, y3, y4;

	/****Side A****/
	coord_conv(box.x2, box.y1, box.z2, &x1, &y1);
	coord_conv(box.x2, box.y1, box.z1, &x2, &y2);
	coord_conv(box.x2, box.y2, box.z1, &x3, &y3);
	coord_conv(box.x2, box.y2, box.z2, &x4, &y4);

	fill_polygon(screen16, x1, y1, x2, y2, x3, y3, x4, y4, box.colorA);

	/****Side B****/
	coord_conv(box.x2, box.y1, box.z2, &x1, &y1);
	coord_conv(box.x1, box.y1, box.z2, &x2, &y2);
	coord_conv(box.x1, box.y2, box.z2, &x3, &y3);
	coord_conv(box.x2, box.y2, box.z2, &x4, &y4);

	fill_polygon(screen16, x1, y1, x2, y2, x3, y3, x4, y4, box.colorB);

	/****Side C****/
	coord_conv(box.x2, box.y2, box.z2, &x1, &y1);
	coord_conv(box.x1, box.y2, box.z2, &x2, &y2);
	coord_conv(box.x1, box.y2, box.z1, &x3, &y3);
	coord_conv(box.x2, box.y2, box.z1, &x4, &y4);

	fill_polygon(screen16, x1, y1, x2, y2, x3, y3, x4, y4, box.colorC);

	// White lines
	draw_line(screen16, coord_convX(box.x2, box.y1, box.z2), coord_convY(box.x2, box.y1, box.z2), coord_convX(box.x2, box.y2, box.z2), coord_convY(box.x2, box.y2, box.z2), 0xFFFF);	//line 7
	draw_line(screen16, coord_convX(box.x2, box.y2, box.z1), coord_convY(box.x2, box.y2, box.z1), coord_convX(box.x2, box.y2, box.z2), coord_convY(box.x2, box.y2, box.z2), 0xFFFF);	//line 8
	draw_line(screen16, coord_convX(box.x1, box.y2, box.z2), coord_convY(box.x1, box.y2, box.z2), coord_convX(box.x2, box.y2, box.z2), coord_convY(box.x2, box.y2, box.z2), 0xFFFF);	//line 9

	// Black lines
	draw_line(screen16, coord_convX(box.x2, box.y1, box.z2), coord_convY(box.x2, box.y1, box.z2), coord_convX(box.x2, box.y1, box.z1), coord_convY(box.x2, box.y1, box.z1), 0x0000);	//line 1
	draw_line(screen16, coord_convX(box.x2, box.y2, box.z1), coord_convY(box.x2, box.y2, box.z1), coord_convX(box.x2, box.y1, box.z1), coord_convY(box.x2, box.y1, box.z1), 0x0000);	//line 2
	draw_line(screen16, coord_convX(box.x2, box.y2, box.z1), coord_convY(box.x2, box.y2, box.z1), coord_convX(box.x1, box.y2, box.z1), coord_convY(box.x1, box.y2, box.z1), 0x0000);	//line 3
	draw_line(screen16, coord_convX(box.x1, box.y2, box.z2), coord_convY(box.x1, box.y2, box.z2), coord_convX(box.x1, box.y2, box.z1), coord_convY(box.x1, box.y2, box.z1), 0x0000);	//line 4
	draw_line(screen16, coord_convX(box.x1, box.y2, box.z2), coord_convY(box.x1, box.y2, box.z2), coord_convX(box.x1, box.y1, box.z2), coord_convY(box.x1, box.y1, box.z2), 0x0000);	//line 5
	draw_line(screen16, coord_convX(box.x2, box.y1, box.z2), coord_convY(box.x2, box.y1, box.z2), coord_convX(box.x1, box.y1, box.z2), coord_convY(box.x1, box.y1, box.z2), 0x0000);	//line 6
}

void ball_shadow(uint16_t *screen16, uint32_t *bg, ball_t ball)
{
	int32_t x, x2, y;
	uint16_t red, green, blue;

	if (REAL_SHADOW == 1)
		coord_conv(ball.x + (ball.y>>1), 0, ball.z, &x, &y);
	else
		coord_conv(ball.x/* + (ball.y>>1)*/, 0, ball.z, &x, &y);

	x2 = x;
	
	if (y>=0 && y<SCREEN_H)
	{
		for (x=x2-1; x<x2+1; x++)
			if (x>=0 && x<SCREEN_W)
			{
				red = ((bg[y * SCREEN_W + x] >> 16) & 0xF800) >> 8;	//turns values in 8-bit color values
				green = ((bg[y * SCREEN_W + x] >> 16) & 0x07E0) >> 3;
				blue = ((bg[y * SCREEN_W + x] >> 16) & 0x001F) << 3;

				red = red * SHADOW_INTENS >> 8;
				green = green * SHADOW_INTENS >> 8;
				blue = blue * SHADOW_INTENS >> 8;

				screen16[y * SCREEN_W + x] = ((red<<8) & 0xF800) | ((green<<3) & 0x07E0) | (blue>>3);
			}
	}
}

void box_shadow(uint16_t *screen16, box_t box)
{
	int32_t iy, ix, x1, x2, x3, x4, y1, y2, y3, y4;
	uint16_t red, green, blue;

	coord_conv(box.x2 + (box.y2>>1), 0, box.z2, &x1, &y1);
	coord_conv(box.x2 + (box.y2>>1), 0, box.z1, &x2, &y2);
	coord_conv(box.x1 + (box.y1>>1), 0, box.z1, &x3, &y3);
	coord_conv(box.x1 + (box.y1>>1), 0, box.z2, &x4, &y4);

	fa.y_offset = y1;	// looking for the top point
	if (y2 < fa.y_offset)
		fa.y_offset = y2;
	if (y3 < fa.y_offset)
		fa.y_offset = y3;
	if (y4 < fa.y_offset)
		fa.y_offset = y4;

	fa.y_end = y1;	// looking for the lowest point
	if (y2 > fa.y_end)
		fa.y_end = y2;
	if (y3 > fa.y_end)
		fa.y_end = y3;
	if (y4 > fa.y_end)
		fa.y_end = y4;

	fa.y_end++;

	if (fa.y_end > SCREEN_H)
		fa.y_end = SCREEN_H;

	for (iy=fa.y_offset; iy<fa.y_end; iy++)
	{
		fa.line[iy].xmin = SCREEN_W;
		fa.line[iy].xmax = -1;
	}

	draw_line_array(x1, y1, x2, y2);
	draw_line_array(x3, y3, x2, y2);
	draw_line_array(x3, y3, x4, y4);
	draw_line_array(x1, y1, x4, y4);

	for (iy=fa.y_offset; iy<fa.y_end; iy++)
	{
		if (iy>=0 && iy<SCREEN_H)
			for (ix=fa.line[iy].xmin; ix<=fa.line[iy].xmax; ix++)
			{
				if (ix>=0 && ix<SCREEN_W)
				{
					red = (screen16[iy * SCREEN_W + ix] & 0xF800) >> 8;	// turns values in 8-bit color values
					green = (screen16[iy * SCREEN_W + ix] & 0x07E0) >> 3;
					blue = (screen16[iy * SCREEN_W + ix] & 0x001F) << 3;
		
					red = red * SHADOW_INTENS >> 8;
					green = green * SHADOW_INTENS >> 8;
					blue = blue * SHADOW_INTENS >> 8;
		
					screen16[iy * SCREEN_W + ix] = ((red<<8) & 0xF800) | ((green<<3) & 0x07E0) | (blue>>3);
				}
			}
	}
}

void box_ground_mov(box_t *box, int32_t ms)
{
	//static int32_t stop_time=0;

	if (box->left == 0 && box->right == 0)
		box->moving = 0;

	if (box->left == 1)
	{
		if (box->moving == 0)
			box->moving = 1;

		if (box->moving == 1)	// if left leg is moving
		{
			box->x2 += BOX_SPD * ms >> 16;

			if (box->x2 - box->x1 > BOX_MAX)
			{
				box->moving = 2;	//change of moving leg
				box->x2 = BOX_MAX + box->x1;
			}
		}

		if (box->moving == 2)	// if right leg is moving
		{
			box->x1 += BOX_SPD * ms >> 16;

			if (box->x2 - box->x1 < BOX_MIN)
			{
				box->moving = 1;	//change of moving leg
				box->x2 = BOX_MIN + box->x1;
			}
		}
	}

	if (box->right == 1)
	{
		if (box->moving == 0)
			box->moving = 2;

		if (box->moving == 2)	// if right leg is moving
		{
			box->x1 -= BOX_SPD * ms >> 16;
	
			if (box->x2 - box->x1 > BOX_MAX)
			{
				box->moving = 1;	//change of moving leg
				box->x1 = box->x2 - BOX_MAX;
			}
		}

		if (box->moving == 1)	// if left leg is moving
	{
		box->x2 -= BOX_SPD * ms >> 16;

			if (box->x2 - box->x1 < BOX_MIN)
			{
				box->moving = 2;	//change of moving leg
				box->x2 = BOX_MIN + box->x1;
			}
		}
	}

	/*if (box->moving == 0)
		stop_time += ms;
	else
		stop_time = 0;

	if (stop_time > (1<<16))	// (1<<16) = roughly 1 second
	{
		box->x2 += BOX_STOPSPD * ms >> 17;
		box->x1 -= BOX_STOPSPD * ms >> 17;
		box->speed.x = 0;
		if (box->x2 - box->x1 < BOX_MIN)
		{
			box->x1 = box->g.x - (BOX_MIN >> 1);
			box->x2 = box->g.x + (BOX_MIN >> 1);
		}
	}*/

	box->y2 = (BOX_MAX * BOX_MIN) / (box->x2 - box->x1);
}

void box_jump(box_t *box, int32_t ms, int32_t tick)
{
	box->g.x = box->x1 + box->x2 >> 1;
	box->g.y = box->y1 + box->y2 >> 1;

	box->geo_speed = BOX_FLYSPD;

	if (box->left == 0 && box->right == 0)	// if box is standing still
	{
		box->x2 -= BOX_JMPSPD * ms >> 17;
		box->x1 += BOX_JMPSPD * ms >> 17;
		//box->speed.x = 0;
		if (box->x2 - box->x1 < BOX_MIN)
		{
			box->x1 = box->g.x - (BOX_MIN >> 1);
			box->x2 = box->g.x + (BOX_MIN >> 1);
		}
	}

	if (box->left == 1)
	{
		// right leg is moving
		box->x1 += BOX_JMPSPD * ms >> 16;
		//box->speed.x = BOX_JMPSPD >> 1;

		if (box->x2 - box->x1 < BOX_MIN)
			box->x2 = BOX_MIN + box->x1;
	}

	if (box->right == 1)
	{
		// left leg is moving
		box->x2 -= BOX_JMPSPD * ms >> 16;
		//box->speed.x = -BOX_JMPSPD >> 1;

		if (box->x2 - box->x1 < BOX_MIN)
			box->x2 = BOX_MIN + box->x1;
	}

	box->y2 = (BOX_MAX * BOX_MIN) / (box->x2 - box->x1);

	box->speed.y = box->y2 * (BOX_JMPSPD >> 1) / (box->x2 - box->x1);	// ex : 13 * 10 / 5 = 26.0   other ex : 5 * 10 / 13 = 3.84

	if (box->y2 == BOX_MAX)
		box->flying = 3;		// if box is stretched upwards, enter flight mode

	box->g.x = box->x1 + box->x2 >> 1;
	box->g.y = box->y1 + box->y2 >> 1;

	if (tick != box->pre_jump_time)		// if we're not at the same frame as the start of the jump
	{
		box->speed.x = (box->g.x - box->pre_jump_g_x << 16) / ((tick - box->pre_jump_time) * 66);	// calculation of the lateral speed when jumping
		if (box->g.x == box->pre_jump_g_x)			// if the box hasn't moved sideways
		{
			if (box->left==1)				// but that the player/AI is trying to move to the left
				box->speed.x = BOX_JMPSPD >> 1;		// jump to the left
			if (box->right==1)				// and if the player/AI is trying to move to the right
				box->speed.x = -BOX_JMPSPD >> 1;	// jump to the right
		}
		if (box->speed.x < -BOX_JMPSPD >> 1)
			box->speed.x = -BOX_JMPSPD >> 1;
		if (box->speed.x > BOX_JMPSPD >> 1)
			box->speed.x = BOX_JMPSPD >> 1;
	}
	else
	{
		box->speed.x = 0;

		if (box->left==1)				// the player/AI is trying to move to the left
			box->speed.x = BOX_JMPSPD >> 1;		// jump to the left
		if (box->right==1)				// and if the player/AI is trying to move to the right
			box->speed.x = -BOX_JMPSPD >> 1;	// jump to the right
	}
}

void box_fly(box_t *box, int32_t ms)
{
	box->size.x = box->x2 - box->x1;
	box->size.y = box->y2 - box->y1;

	box->g.x = box->x1 + box->x2 >> 1;
	box->g.y = box->y1 + box->y2 >> 1;

	box->speed.y += (int32_t) ms * (int32_t) (-GRAV * (float) (1<<FRAC)) >> 16;
	box->g.y += (int32_t) (box->speed.y * ms) >> 16;
	box->g.x += (int32_t) (box->speed.x * ms) >> 16;

	box->y2 = box->g.y + (box->size.y >> 1);
	box->y1 = box->g.y - (box->size.y >> 1);

	box->x2 = box->g.x + (box->size.x >> 1);
	box->x1 = box->g.x - (box->size.x >> 1);

	// GEOMETRY
	
	/*box->x2 -= box->geo_speed * ms >> 17;
	box->x1 += box->geo_speed * ms >> 17;
	if (box->x2 - box->x1 < BOX_MIN)
	{
		box->x1 = box->g.x - (BOX_MIN >> 1);
		box->x2 = box->g.x + (BOX_MIN >> 1);
		box->geo_speed = -BOX_FLYSPD;
	}
	if (box->x2 - box->x1 > BOX_MAX)
	{
		box->x1 = box->g.x - (BOX_MAX >> 1);
		box->x2 = box->g.x + (BOX_MAX >> 1);
		box->geo_speed = BOX_FLYSPD;
	}

	box->size.y = (BOX_MAX * BOX_MIN) / (box->x2 - box->x1);
	box->y2 = box->g.y + (box->size.y >> 1);
	box->y1 = box->g.y - (box->size.y >> 1);*/

	box->y2 -= box->geo_speed * ms >> 17;
	box->y1 += box->geo_speed * ms >> 17;
	if (box->y2 - box->y1 < BOX_MIN)
	{
		box->y1 = box->g.y - (BOX_MIN >> 1);
		box->y2 = box->g.y + (BOX_MIN >> 1);
		box->geo_speed = -BOX_FLYSPD;
	}
	if (box->y2 - box->y1 > BOX_MAX)
	{
		box->y1 = box->g.y - (BOX_MAX >> 1);
		box->y2 = box->g.y + (BOX_MAX >> 1);
		box->geo_speed = BOX_FLYSPD;
	}

	box->size.x = (BOX_MAX * BOX_MIN) / (box->y2 - box->y1);
	box->x2 = box->g.x + (box->size.x >> 1);
	box->x1 = box->g.x - (box->size.x >> 1);

	if (box->y1 <= 0)
	{
		box->y1 = 0;
		box->flying = 4;	// landing
	}

/*if (box->g.y > max_h)
	max_h = box->g.y;*/
}

void box_land(box_t *box, int32_t ms)
{
	box->g.x = box->x1 + box->x2 >> 1;
	box->g.y = box->y1 + box->y2 >> 1;

	box->x2 -= -BOX_LANDSPD * ms >> 17;
	box->x1 += -BOX_LANDSPD * ms >> 17;
	if (box->x2 - box->x1 > BOX_MAX)
	{
		box->x1 = box->g.x - (BOX_MAX >> 1);
		box->x2 = box->g.x + (BOX_MAX >> 1);
		box->flying = 0;
	}

	box->size.y = (BOX_MAX * BOX_MIN) / (box->x2 - box->x1);
	box->y2 = box->size.y;
	box->y1 = 0;
}


void ball_throw(ball_t *ball, box_t box, int32_t tgt_a)
{
	//int32_t tgt_a = rand()%LUT_AC;
	int32_t rhc = -(ball->y>>FRAC) + 32 + ((BOX_MIN>>1)+1>>FRAC);
	int32_t rdc = 0;
	int32_t rma;

	if ((box.flying == 1 || box.flying == 3) && box.speed.y > 0 && ball->y <= BOX_MIN && service==0)	// throwing a high ball
	{
		rhc = 32 + HIBALL_TGT_ALT;
		ball_jump = 1;
		if (tgt_a < TGT_ANGLE_MIN)
			tgt_a = TGT_ANGLE_MIN;	// Min. angle (otherwise the ball goes to fast to be caught)
	}
	else
		ball_jump = 0;

	if (ball->y > BOX_MAX)
		tgt_a = 0;

	int32_t d;
	//int32_t rx = ((rand()%28) - 14) << FRAC;	// random target X
	int32_t dz = abs(ball->z) + (BOX_D << FRAC);
	int32_t dx = -(tgt_x - ball->x);

	d = distance(0, 0, dx, dz);

	rdc = ((d - LUT_MIN_DST << FRAC) / LUT_DST_STEP) >> FRAC;
	//d = id * LUT_DST_STEP + LUT_MIN_DST
	
	// Out-of-bounds checks
	if (rhc >= LUT_HC)
		rhc = LUT_HC-1;
	if (rdc >= LUT_DC)
		rdc = LUT_DC-1;

	if (rhc<0)
		rhc=0;
	if (rdc<0)
		rdc=0;

	rma = (int32_t) min_angle[rhc][rdc];	// look up the lowest angle tolerable that won't hit the net

	//printf("tgt_a #1 : %d (speed : %d, height : %d, ball.y : %d, rma : %d)\t", tgt_a, lut[rhc][rdc][tgt_a].speed, lut[rhc][rdc][tgt_a].height, ball->y, rma);

	//tgt_a = /*rma + (LUT_AC - rma) * */((tgt_a - rma << 16) / (LUT_AC - rma) >> 0);// * ((tgt_a - rma << 16) / (LUT_AC - rma) >> 16);
	if (tgt_a < rma)
		tgt_a = rma;
	if (tgt_a >= LUT_AC)
		tgt_a = LUT_AC-1;

	while (lut[rhc][rdc][tgt_a].height + ball->y < NET_HEIGHT)
		tgt_a++;


	//printf("tgt_a #2 : %d\n", tgt_a);

	int32_t ball_speed_mag = lut[rhc][rdc][tgt_a].speed;

	int32_t asq = (int32_t) ((float) (1<<LUT_ANG_FMT) * sqrtf(1.0f + powf(2.0f, (float) (tgt_a * LUT_ANG_STEP + LUT_LOW_ANG) / (float) (1<<LUT_ANG_FMT))));	// ?.8 format

	ball->speed.y = ball_speed_mag * (tgt_a * LUT_ANG_STEP + LUT_LOW_ANG) / asq;
	ball->speed.z = -(ball_speed_mag << LUT_ANG_FMT) / asq;		// speed mag without Y

	ball->speed.x = (ball->speed.z * dx) / d;
	ball->speed.z = (ball->speed.z * dz) / d;
}

void col_detection(box_t box, ball_t *ball)
{
	int32_t tgt_a, already_thrown=0;

	if (ball->prev.z > box.z2+BALL_RADIUS && ball->z <= box.z2+BALL_RADIUS && box.name == 1)	// if ball flies by the level of the box1
	{
		ball->col_ratio = ((box.z2+BALL_RADIUS - ball->prev.z) << 16) / (ball->z - ball->prev.z);	// ball->col_ratio is in 0.16 format
		ball->col.x = ((ball->x - ball->prev.x) * ball->col_ratio >> 16) + ball->prev.x;
		ball->col.y = ((ball->y - ball->prev.y) * ball->col_ratio >> 16) + ball->prev.y;

		if (ball->col.x >= box.x1-BALL_RADIUS && ball->col.x <= box.x2+BALL_RADIUS && ball->col.y >= box.y1-BALL_RADIUS && ball->col.y <= box.y2+BALL_RADIUS)	// if there's collision
		{
			hit_sound.pos = 0;
			hit_sound.vol = 65535;
			if (service==1)		// if the previous hit was a service
				service=0;	// set the service bit off

			tgt_x = (((ball->col.x - box.x1) * 2 * COURT_WIDTH) / (box.x2 - box.x1)) - COURT_WIDTH;
			//printf("tgt_x(%d) : %d\tthen", box.name, tgt_x);
			tgt_x = sqrtq(tgt_x) * sign(tgt_x) * sqrtq(COURT_WIDTH)>>FRAC;
			//printf(" %d\n", tgt_x);
			tgt_a = ((ball->col.y - (box.y2+BALL_RADIUS)) * (LUT_AC - 1) / ((box.y1-BALL_RADIUS) - (box.y2+BALL_RADIUS)));
//tgt_a = LUT_AC - 1 - tgt_a;

			ball->z = box.z2+BALL_RADIUS;
			ball->x = ball->col.x;
			ball->y = ball->col.y;
			//ball->speed.z = -ball->speed.z * (int32_t) (1.0f * 256.0f) >> 8;
			ball_throw(ball, box, tgt_a);
			already_thrown = 1;
			hitting_box = 1;
			ball->speed.z = -ball->speed.z;

			fast_sound.pos = 0;
			fast_sound.vol = ball->speed.z * ball->speed.z >> 21;
			fast_sound.vol *= ball->speed.z * ball->speed.z >> 21;
			fast_sound.vol *= ball->speed.z * ball->speed.z >> 21;

			return;
		}
	}

	if (ball->prev.z < box.z1-BALL_RADIUS && ball->z >= box.z1-BALL_RADIUS && box.name == 2)	// if ball flies by the level of the box2
	{
		ball->col_ratio = ((box.z1-BALL_RADIUS - ball->prev.z) << 16) / (ball->z - ball->prev.z);	// ball->col_ratio is in 0.16 format
		ball->col.x = ((ball->x - ball->prev.x) * ball->col_ratio >> 16) + ball->prev.x;
		ball->col.y = ((ball->y - ball->prev.y) * ball->col_ratio >> 16) + ball->prev.y;

		if (ball->col.x >= box.x1-BALL_RADIUS && ball->col.x <= box.x2+BALL_RADIUS && ball->col.y >= box.y1-BALL_RADIUS && ball->col.y <= box.y2+BALL_RADIUS)	// if there's collision
		{
			hit_sound.pos = 0;
			hit_sound.vol = 65535;

			if (service==1)		// if the previous hit was a service
				service=0;	// set the service bit off

			tgt_x = (((ball->col.x - box.x1) * 2 * COURT_WIDTH) / (box.x2 - box.x1)) - COURT_WIDTH;
			//printf("tgt_x(%d) : %d\tthen", box.name, tgt_x);
			tgt_x = sqrtq(tgt_x) * sign(tgt_x) * sqrtq(COURT_WIDTH)>>FRAC;
			//printf(" %d\n", tgt_x);
			tgt_a = ((ball->col.y - box.y2+BALL_RADIUS) * (LUT_AC - 1) / ((box.y1-BALL_RADIUS) - (box.y2+BALL_RADIUS)));
//tgt_a = LUT_AC - 1 - tgt_a;

			ball->z = box.z1-BALL_RADIUS;
			ball->x = ball->col.x;
			ball->y = ball->col.y;
			//ball->speed.z = -ball->speed.z * (int32_t) (1.0f * 256.0f) >> 8;
			ball_throw(ball, box, tgt_a);
			hitting_box = 2;
			already_thrown = 1;

			fast_sound.pos = 0;
			fast_sound.vol = ball->speed.z * ball->speed.z >> 21;
			fast_sound.vol *= ball->speed.z * ball->speed.z >> 21;
			fast_sound.vol *= ball->speed.z * ball->speed.z >> 21;

			return;
		}
	}

	if (already_thrown == 0 && ball->x<=box.x2+BALL_RADIUS && ball->x>=box.x1-BALL_RADIUS && ball->y<=box.y2+BALL_RADIUS && ball->y>=box.y1-BALL_RADIUS && ball->z<=box.z2+BALL_RADIUS && ball->z>=box.z1-BALL_RADIUS)	// if the ball is now in the box
	{
		if (ball->prev.y > box.y2+BALL_RADIUS && ball->y <= box.y2+BALL_RADIUS)		// top face collision
		{
			ball->col_ratio = ((box.y2+BALL_RADIUS - ball->prev.y) << 16) / (ball->y - ball->prev.y);	// ball->col_ratio is in 0.16 format
			ball->col.x = ((ball->x - ball->prev.x) * ball->col_ratio >> 16) + ball->prev.x;
			ball->col.z = ((ball->z - ball->prev.z) * ball->col_ratio >> 16) + ball->prev.z;

			if (ball->col.x >= box.x1-BALL_RADIUS && ball->col.x <= box.x2+BALL_RADIUS && ball->col.z >= box.z1-BALL_RADIUS && ball->col.z <= box.z2+BALL_RADIUS)
			{
				ball->y = box.y2+BALL_RADIUS;
				ball->speed.y = -ball->speed.y;
			}
		}

		if (ball->prev.y < box.y1+BALL_RADIUS && ball->y >= box.y1+BALL_RADIUS)		// bottom face collision
		{
			ball->col_ratio = ((box.y1-BALL_RADIUS - ball->prev.y) << 16) / (ball->y - ball->prev.y);	// ball->col_ratio is in 0.16 format
			ball->col.x = ((ball->x - ball->prev.x) * ball->col_ratio >> 16) + ball->prev.x;
			ball->col.z = ((ball->z - ball->prev.z) * ball->col_ratio >> 16) + ball->prev.z;

			if (ball->col.x >= box.x1-BALL_RADIUS && ball->col.x <= box.x2+BALL_RADIUS && ball->col.z >= box.z1-BALL_RADIUS && ball->col.z <= box.z2+BALL_RADIUS)
			{
				ball->y = box.y1-BALL_RADIUS;
				ball->speed.y = -ball->speed.y;
			}
		}

		if (ball->prev.z < box.z1-BALL_RADIUS && ball->z >= box.z1-BALL_RADIUS)		// back face collision
		{
			ball->col_ratio = ((box.z1-BALL_RADIUS - ball->prev.z) << 16) / (ball->z - ball->prev.z);	// ball->col_ratio is in 0.16 format
			ball->col.x = ((ball->x - ball->prev.x) * ball->col_ratio >> 16) + ball->prev.x;
			ball->col.y = ((ball->y - ball->prev.y) * ball->col_ratio >> 16) + ball->prev.y;
	
			if (ball->col.x >= box.x1-BALL_RADIUS && ball->col.x <= box.x2+BALL_RADIUS && ball->col.y >= box.y1-BALL_RADIUS && ball->col.y <= box.y2+BALL_RADIUS)
			{
				ball->z = box.z1-BALL_RADIUS;
				ball->speed.z = -ball->speed.z;
			}
		}

		if (ball->prev.z > box.z2+BALL_RADIUS && ball->z <= box.z2+BALL_RADIUS)		// front face collision
		{
			ball->col_ratio = ((box.z2+BALL_RADIUS - ball->prev.z) << 16) / (ball->z - ball->prev.z);	// ball->col_ratio is in 0.16 format
			ball->col.x = ((ball->x - ball->prev.x) * ball->col_ratio >> 16) + ball->prev.x;
			ball->col.y = ((ball->y - ball->prev.y) * ball->col_ratio >> 16) + ball->prev.y;
	
			if (ball->col.x >= box.x1-BALL_RADIUS && ball->col.x <= box.x2+BALL_RADIUS && ball->col.y >= box.y1-BALL_RADIUS && ball->col.y <= box.y2+BALL_RADIUS)
			{
				ball->z = box.z2+BALL_RADIUS;
				ball->speed.z = -ball->speed.z;
			}
		}

		if (ball->prev.x < box.x1-BALL_RADIUS && ball->x >= box.x1-BALL_RADIUS)		// right face collision
		{
			ball->col_ratio = ((box.x1-BALL_RADIUS - ball->prev.x) << 16) / (ball->x - ball->prev.x);	// ball->col_ratio is in 0.16 format
			ball->col.y = ((ball->y - ball->prev.y) * ball->col_ratio >> 16) + ball->prev.y;
			ball->col.z = ((ball->z - ball->prev.z) * ball->col_ratio >> 16) + ball->prev.z;
	
			if (ball->col.z >= box.z1-BALL_RADIUS && ball->col.z <= box.z2+BALL_RADIUS && ball->col.y >= box.y1-BALL_RADIUS && ball->col.y <= box.y2+BALL_RADIUS)
			{
				ball->x = box.x1-BALL_RADIUS;
				ball->speed.x = -ball->speed.x;
			}
		}

		if (ball->prev.x > box.x2+BALL_RADIUS && ball->x <= box.x2+BALL_RADIUS)		// left face collision
		{
			ball->col_ratio = ((box.x2+BALL_RADIUS - ball->prev.x) << 16) / (ball->x - ball->prev.x);	// ball->col_ratio is in 0.16 format
			ball->col.y = ((ball->y - ball->prev.y) * ball->col_ratio >> 16) + ball->prev.y;
			ball->col.z = ((ball->z - ball->prev.z) * ball->col_ratio >> 16) + ball->prev.z;
	
			if (ball->col.z >= box.z1-BALL_RADIUS && ball->col.z <= box.z2+BALL_RADIUS && ball->col.y >= box.y1-BALL_RADIUS && ball->col.y <= box.y2+BALL_RADIUS)
			{
				ball->x = box.x2+BALL_RADIUS;
				ball->speed.x = -ball->speed.x;
			}
		}	
	}
}

void box_mov(box_t *box, int32_t ms, int32_t tick)
{
	if (box->jump==1)
		switch (box->flying)
		{
			case 0:			// if box is on the ground
				box->flying=1;	// jump
				box->pre_jump_g_x = box->x1 + box->x2 >> 1;
				box->pre_jump_time = tick;
				break;
			case 2:			// if jump was aborpted
				box->flying=1;	// TODO FIXME
				break;
			case 4:			// if box is on the ground
				//box->flying=1;	// TODO pretty much same as case 2
				break;
		}

	if (box->jump==0)
		switch (box->flying)
		{
			case 1:			// if box is jumping
				box->flying=3;	// aborpt
				break;
		}

	switch (box->flying)
	{	
		// if box is on the ground, not jumping or anything
		case 0:
			box_ground_mov(box, ms);
			break;

		// if box is jumping
		case 1:
			box_jump(box, ms, tick);
			break;
		// if jump has been aborpted
		case 2:		// TODO
		// if box is flying
		case 3:
			box_fly(box, ms);
			break;
		case 4:
			box_land(box, ms);
			break;
	}
}


lut_entry ballistic_test(int32_t speed, int32_t angle, int32_t distance)	// Formats : 10.6, 0.8, 10.6
{
	ball_t ball;
	int32_t asq;
	lut_entry o;	// o.speed == final Y coord. o.height = net level Y coord.

i++;

	ball.x = 0;
	ball.y = 0;
	ball.z = 0;
	ball.speed.x = 0;

	asq = (int32_t) ((float) (1<<LUT_ANG_FMT) * sqrtf(1.0f + powf(2.0f, (float) angle / (float) (1<<LUT_ANG_FMT))));	// ?.8 format

	ball.speed.y = speed * angle / asq;
	ball.speed.z = (speed << LUT_ANG_FMT) / asq;

	while (ball.z < distance)	// fly the ball until the distance is reached
	{
		ball.prev.x = ball.x;
		ball.prev.y = ball.y;
		ball.prev.z = ball.z;

		// Gravity
		ball.speed.y += (int32_t) LUT_MS * (int32_t) (-GRAV * (float) (1<<FRAC)) >> 16;	// gravity in m/s^2 (97.4 to compensate the 65.536/66 imprecision)

		// Air drag
		ball.speed.y -= sign(ball.speed.y) * (int32_t) LUT_MS * ((((ball.speed.y * ball.speed.y)>>16) * (int32_t) ((float) (1 << 10+12) * GRAV / (BALL_TV * BALL_TV))) >> 16) >>  12;
		ball.speed.z -= sign(ball.speed.z) * (int32_t) LUT_MS * ((((ball.speed.z * ball.speed.z)>>16) * (int32_t) ((float) (1 << 10+12) * GRAV / (BALL_TV * BALL_TV))) >> 16) >>  12;

		// Position calculation
		ball.y += (int32_t) (ball.speed.y * LUT_MS) >> 16;
		ball.z += (int32_t) (ball.speed.z * LUT_MS) >> 16;

		if (ball.prev.z < (distance>>1) && ball.z > (distance>>1))	// net height test
			o.height = (int16_t) (int32_t) ((ball.y - ball.prev.y) * (((distance - ball.prev.z) << 16) / (ball.z - ball.prev.z)) >> 16) + ball.prev.y;

	}
	
	// when the distance is reached, calculate where that distance has been reached
	ball.col_ratio = ((distance - ball.prev.z) << 16) / (ball.z - ball.prev.z);	// ball->col_ratio is in 0.16 format

	o.speed = ((ball.y - ball.prev.y) * ball.col_ratio >> 16) + ball.prev.y;	// Y coordinate of the ball

	return o;
}

void ballistics_lut()
{
	FILE *lut_file;
	int32_t l, ia, ih, id, max_spd, min_spd, tgt_y, y_error, speed, word, match;
	float ball_tv = BALL_TV, grav=GRAV;
	lut_entry test_y;

	int32_t lut_macro[] = {*(int32_t *) &ball_tv, *(int32_t *) &grav, FAILURE_SPD, BOX_D, BOX_MAXJMP, LUT_MS, LUT_ERR, LUT_DC, LUT_DST_STEP, LUT_MIN_SPD, LUT_MAX_SPD, LUT_START_SPD, LUT_ANG_FMT, LUT_LOW_ANG, LUT_HIG_ANG, LUT_AC, LUT_ANG_STEP, LUT_MIN_DST, LUT_MIN_HEI, LUT_HEI_STEP, LUT_HC};

	lut_file = fopen("ballistics_lut.dat", "rb");

	match = 0;

		lut = malloc(LUT_HC * sizeof(lut_entry **));
		if (lut == NULL)
			fprintf(stderr, "malloc returned null on lut\n");
		for (ih=0; ih<LUT_HC; ih++)
		{
			lut[ih] = malloc(LUT_DC * sizeof(lut_entry *));
			if (lut[ih] == NULL)
				fprintf(stderr, "malloc returned null on lut[%d]\n", ih);
			for (id=0; id<LUT_DC; id++)
			{
				lut[ih][id] = calloc(LUT_AC, sizeof(lut_entry));
				if (lut[ih][id] == NULL)
					fprintf(stderr, "malloc returned null on lut[%d][%d]\n", ih, id);
			}
		} 

		min_angle = malloc(LUT_HC * sizeof(int16_t *));
		if (min_angle == NULL)
			fprintf(stderr, "malloc returned null on min_angle\n");
		for (ih=0; ih<LUT_HC; ih++)
		{
			min_angle[ih] = calloc(LUT_DC, sizeof(int16_t));
			if (min_angle[ih] == NULL)
				fprintf(stderr, "malloc returned null on min_angle[%d]\n", ih);
		}

	if (lut_file)	// if LUT file exists
	{
		match = 1;

		for (l = 0; l < sizeof(lut_macro) / sizeof(int32_t); l++)	// comparing written macros
		{
			fread(&word, 1, sizeof(int32_t), lut_file);

			if (word != lut_macro[l])
				match = 0;
		}

		if (match == 1)
		{
		printf("Ballistics look-up table loading...");

		// LUT loading
			for (ih=0; ih<LUT_HC; ih++)
			{
				for (id=0; id<LUT_DC; id++)
				{
					fread(&min_angle[ih][id], 1, sizeof(int16_t), lut_file);
					for (ia=0; ia<LUT_AC; ia++)
					{
						fread(&lut[ih][id][ia].speed, 1, sizeof(int32_t), lut_file);
						fread(&lut[ih][id][ia].height, 1, sizeof(int16_t), lut_file);
					}
				}
			}
		}

		fclose(lut_file);
	}
	if (match==0)		// if LUT is to be generated
	{
		printf("Ballistics look-up table generation...");

		lut_file = fopen("ballistics_lut.dat", "wb");

		for (l = 0; l < sizeof(lut_macro) / sizeof(int32_t); l++)	// writting macros
			fwrite(&lut_macro[l], 1, sizeof(int32_t), lut_file);

		for (ih=0; ih<LUT_HC; ih++)
		{
			//fprintf(stderr, "%d :\n", ih);
			for (id=0; id<LUT_DC; id++)
			{
				tgt_y = ih * LUT_HEI_STEP + LUT_MIN_HEI;
				for (ia=0; ia<LUT_AC; ia++)
				{
					max_spd = LUT_MAX_SPD;
					min_spd = LUT_MIN_SPD;
					speed = LUT_START_SPD;
					y_error = LUT_ERR<<1;
	
					if (ballistic_test(max_spd, ia * LUT_ANG_STEP + LUT_LOW_ANG, id * LUT_DST_STEP + LUT_MIN_DST).speed > tgt_y && ballistic_test(min_spd, ia * LUT_ANG_STEP + LUT_LOW_ANG, id * LUT_DST_STEP + LUT_MIN_DST).speed < tgt_y)
					{
						while (abs(y_error) >= LUT_ERR)
						{
							test_y = ballistic_test(speed, ia * LUT_ANG_STEP + LUT_LOW_ANG, id * LUT_DST_STEP + LUT_MIN_DST);
	
							y_error = test_y.speed - tgt_y;
		
							if (abs(y_error) >= LUT_ERR)
							{
								if (y_error > 0)
									max_spd = speed;
								if (y_error < 0)
									min_spd = speed;
		
								speed = (max_spd - min_spd >> 1) + min_spd;	// new speed is in the middle of the new search range
							}
						}
					}
					else
						speed = FAILURE_SPD;
	
					lut[ih][id][ia].speed = speed;
					lut[ih][id][ia].height = test_y.height;
					//fprintf(stderr, "%d (%d)\t", lut[ih][id][ia].speed, lut[ih][id][ia].height);
	if (speed!=FAILURE_SPD) { foobar++; somme += speed;}
				}
				//fprintf(stderr, "\n");
			}
			//fprintf(stderr, "\n\n");
		}

		// Minimum angle finding
		for (ih=0; ih<LUT_HC; ih++)
		{
			for (id=0; id<LUT_DC; id++)
			{
				ia = 0;
				min_angle[ih][id] = 0;

				while (lut[ih][id][ia].speed == FAILURE_SPD)	// there shall ALWAYS be a valid angle available
				{
					min_angle[ih][id] = ia + 1;
					ia++;
				}
			}
		}

		// LUT saving
		for (ih=0; ih<LUT_HC; ih++)
		{
			for (id=0; id<LUT_DC; id++)
			{
				fwrite(&min_angle[ih][id], 1, sizeof(int16_t), lut_file);
				for (ia=0; ia<LUT_AC; ia++)
				{
					fwrite(&lut[ih][id][ia].speed, 1, sizeof(int32_t), lut_file);
					fwrite(&lut[ih][id][ia].height, 1, sizeof(int16_t), lut_file);
				}
			}
		}

		fclose(lut_file);
	}

	printf("Done\n");
}

void score_blit(uint16_t *screen, uint8_t *sprite, uint16_t width, uint16_t height, int32_t x, int32_t y)
{
	int32_t iy, ix;
	uint8_t pixel;

	for (iy=y; iy<y+height; iy++)
	{
		for (ix=x; ix<x+width; ix++)
		{
			pixel = sprite[(iy - y) * width + (ix - x)];
			if (pixel != 0)
				screen[iy * SCREEN_W + ix] = 0xffff;
		}
	}
}

#ifdef GP2X
void fb_scale(uint16_t *a, uint16_t *b)
{
	int32_t iy, ix;

	for (iy=0; iy<SCREEN_H; iy++)
	{
		for (ix=0; ix<SCREEN_W; ix++)
		{
			b[(iy << 1) * 320 + (ix << 1) + (320 - (SCREEN_W<<1) >> 1)] = a[iy * SCREEN_W + ix];
			b[(iy << 1) * 320 + (ix << 1) + 1 + (320 - (SCREEN_W<<1) >> 1)] = a[iy * SCREEN_W + ix];
			b[((iy << 1) + 1) * 320 + (ix << 1) + (320 - (SCREEN_W<<1) >> 1)] = a[iy * SCREEN_W + ix];
			b[((iy << 1) + 1) * 320 + (ix << 1) + 1 + (320 - (SCREEN_W<<1) >> 1)] = a[iy * SCREEN_W + ix];
			//b[iy * 320 +ix] = a[iy * SCREEN_W + ix];
		}
	}
}

/*void SetClock(unsigned int MHZ)
{
	unsigned int v;
	  	unsigned int mdiv,pdiv=3,scale=0;
	MHZ*=1000000;
	mdiv=(MHZ*pdiv)/7372800;//SYS_CLK_FREQ;
	
	mdiv=((mdiv-8)<<8) & 0xff00;
	pdiv=((pdiv-2)<<2) & 0xfc;
	scale&=3;
	v = mdiv | pdiv | scale;
	
	unsigned int l = memregs32[0x808>>2];// Get interupt flags
	memregs32[0x808>>2] = 0xFF8FFFE7;   // Turn off interrupts
	memregs16[0x910>>1]=v;			  // Set frequency
	while(memregs16[0x0902>>1] & 1);	// Wait for the frequency to be ajused
	memregs32[0x808>>2] = l;			//Turn on interrupts
}*/

void CPUSetting(int32_t speed)
{
	unsigned short mdiv = 0;
	unsigned short pdiv = 3;
	unsigned short scale = 0;
	unsigned short wrt;
	static int last_speed = -1;
	int adjusted;
	
	if(speed == last_speed)
		return;
	
	mdiv = (pdiv * speed*1000*1000) / SYS_CLK_FREQ;
	adjusted = (mdiv * SYS_CLK_FREQ) / (pdiv * 1000*1000);
	mdiv = ((mdiv-8)<<8) & 0xff00;
	pdiv = ((pdiv-2)<<2) & 0xfc;
	scale &= 3;
	wrt = mdiv | pdiv | scale;

	memregs16[FPLLSETVREG>>1] = wrt; // put new value in memory
	printf ("Clock Adjust : %d MHz(%d MHz)\n", speed, adjusted);
	last_speed = speed;
}

/*void hw_set_cpu(int speed)
{
	int m = 0;
	// Calculating the FPLL value from frequency is a bit complex
	// so we just use some known-to-work values instead.
	switch(speed)
	{
	case 250: m = 0x5D04; break;
	case 225: m =  0x5304; break;
	case 200: m = 0x4904; break;
	case 175: m = 0x3F04; break;
	case 150: m = 0x4901; break;
	case 125: m = 0x3c01; break;
	case 100: m = 0x6502; break;
	case 75: m = 0x4902; break;
	case 50: m = 0x6503; break;
	}

	if(m)
	{
		// Get interupt flags
		unsigned int l = memregs32[0x808>>2];

		// Turn off interrupts
		memregs32[0x808>>2] = 0xFF8FFFE7;

		// Set new clock frequency
		memregs16[0x910>>1]=m;

		// Wait for it to take
		while(memregs16[0x0902>>1] & 1);

		// Turn on interrupts again
		memregs32[0x808>>2] = l;
	}
}*/

#endif

inline uint16_t fread_le_short(FILE *file)		// read from file a 16-bit integer in little endian
{
	uint8_t	byte_a, byte_b;

	fread(&byte_a, 1, 1, file);
	fread(&byte_b, 1, 1, file);

	return (uint16_t) (byte_b<<8) | byte_a;
}

inline uint32_t fread_le_word(FILE *file)		// read from file a 32-bit integer in little endian
{
	uint8_t	byte_a, byte_b, byte_c, byte_d;

	fread(&byte_a, 1, 1, file);
	fread(&byte_b, 1, 1, file);
	fread(&byte_c, 1, 1, file);
	fread(&byte_d, 1, 1, file);

	return (uint32_t) (byte_d<<24) | (byte_c<<16) | (byte_b<<8) | byte_a;
}

int16_t *wav_in(char *wavfile, int32_t *samplecount)
{
	FILE *file;
	int32_t i;
	int16_t *sound, *cut_sound;
	int32_t tag[13];
	int32_t zero_start=0, zero_end=0;
	uint8_t byte;

	#ifdef DEBUG
	printf("wav_in...\n");
	#endif

	file = fopen(wavfile, "rb");
	if (file==NULL)
	{
		fprintf(stderr, "The sound file %s could not be found\nExiting with error.\n", wavfile);
		exit(EXIT_FAILURE);
	}

	for (i=0; i<13; i++)					// tag reading
	{
		tag[i]=0;

		if ((i==5) || (i==6) || (i==9) || (i==10))
			tag[i] = fread_le_short(file);
		else
			tag[i] = fread_le_word(file);
	}

	//********File format checking********

	if (tag[0]!=1179011410 || tag[2]!=1163280727)
	{
		fprintf(stderr, "This file is not in WAVE format\n");
		exit(EXIT_FAILURE);
	}

	if (tag[3]!=544501094 || tag[4]!=16 || tag[11]!=1635017060)
	{
		fprintf(stderr, "This WAVE file format is not currently supported\n");
		exit(EXIT_FAILURE);
	}
	//--------File format checking--------

	if (tag[6]!=1)
	{
		fprintf(stderr, "Only mono sounds are supported\n");
		exit(EXIT_FAILURE);
	}

	*samplecount = tag[12]/(tag[10]/8);

	sound = malloc (*samplecount * sizeof(int16_t));

	//********Data loading********

	if (tag[10]==8)
		for (i=0; i<*samplecount; i++)
		{
			fread(&byte, 1, 1, file);
			sound[i] = ((int16_t) byte - 128) << 8;
		}
	if (tag[10]==16)
		for (i=0; i<*samplecount; i++)
			sound[i] = (int16_t) fread_le_short(file);
	if (tag[10]!=8 && tag[10]!=16)
	{
		fprintf(stderr, "Only 8-bit and 16-bit WAVs are currently supported\n");
		exit(EXIT_FAILURE);
	}
	//--------Data loading--------

	//********Zero-padding removal********
	for (i=0; i<*samplecount; i++)
	{
		if (sound[i]!=0 && zero_start==0)
		{
			zero_start = i;
			break;
		}
	}

	for (i=*samplecount-1; i>=0; i--)
	{
		if (sound[i]!=0 && zero_end==0)
		{
			zero_end = i;
			break;
		}
	}

	*samplecount = zero_end - zero_start + 1;

	cut_sound = malloc (*samplecount * sizeof(int16_t));

	memcpy(cut_sound, &sound[zero_start], *samplecount * sizeof(int16_t));

	free(sound);

	fclose(file);
	return cut_sound;
}

void stream_sound(sound_t *sound, uint8_t *stream)
{
	int32_t i, start=0;
	int32_t mix_vol;	// vol and vol_mod combined

	if (sound->pos < 0)				// if the sound is scheduled to play in the future
		if (sound->pos + SOUND_BUFFERLEN < 0)	// and that it won't play during this time chunk
			return;				// quit the function FIXME
		else
		{					// if it will play
			start = -sound->pos;		// skip the silence/out-of-bounds part
			sound->pos = 0;			// and make it start at its beginning
		}

	if (sound->pos < sound->len)
	{
		if (sound->vol * sound->vol_mod >> 24)		// if vol and vol_mod combined are stronger than 1.0
			sound->vol = (1L<<24) / sound->vol_mod;	// cap vol

		mix_vol = sound->vol * sound->vol_mod >> 8;	// mixing volume in 0.16 format

		for (i=start; i<SOUND_BUFFERLEN && sound->pos < sound->len; i++, sound->pos++)	// TODO use a 'stop' variable
		{
			((int16_t *)stream)[i] += (int16_t) ((int32_t) sound->snd[sound->pos] * mix_vol >> 16);
			//((int16_t *)stream)[i] += (int16_t) (((int32_t) sound->snd[sound->pos] * sound->vol >> 16) * sound->vol_mod >> 8);	// old formula that easily caused overflows when vol > 65536
		}
	}
}

void audio_callback(void *userdata, uint8_t *stream, int32_t len)
{
	int32_t i;

	memset(stream, 0, SOUND_BUFFERLEN * sizeof(int16_t));

	stream_sound(&bounce_sound, stream);
	stream_sound(&hit_sound, stream);
	stream_sound(&fast_sound, stream);
}


int main(int argc, char *argv[])
{
	SDL_Surface *screen;
	SDL_Event event;
	#ifdef GP2X
	uint32_t io_word, prev_io=0xFFFFFFFF;
	#endif
	int32_t tick, tick_prev, ms, ball_loss_time=0;
	uint16_t *screen16, *screen16fb, *screen_scaled;
	uint8_t trash1;
	int8_t exit_key=0, reset_key=0, break_key=0, d_key=0, ball_up=0, ball_down=0, ball_left=0, ball_right=0, ball_throwup=0, kp8=0, kp4=0, kp5=0, kp6=0, leftAIkey=0, rightAIkey=0;
	/*SCORING*/
	int8_t scoring=0;
	int8_t player_a, player_b;
	int8_t point[] = {0, 0};	// game for each player, 0 = 0, 1 = 15, 2 = 30, 3 = 40, 4 = A or 40+
	int8_t game[] = {0, 0};		// sets, from 0 to 6
	int8_t set[] = {0, 0};		// won sets
	char *point_codes[] = {"0", "15", "30", "40", "A"};
	uint8_t *point_sprite[5];
	uint16_t point_h, point_w;
	uint8_t *score_sprite[10];
	uint16_t score_h, score_w;
	char filename[12];
	/*END SCORING*/
	/*SPRITES*/
	object_t bg;
	ball_t ball;
	box_t box1, box2;
	/*END SPRITES*/
	/*FONT*/
	uint8_t *font;
	uint16_t font_w, font_h;
	/*END FONT*/
	char *line1=calloc (54, sizeof(char));
	int32_t cpu_speed = 80, cpu_down=0, cpu_up=0;

	font = load_png_file("tinyfont.png", &font_w, &font_h, &trash1, 2);

	srand(time(NULL));

	//printf("HC : %d\tDC : %d\tAC : %d\n", LUT_HC, LUT_DC, LUT_AC);

	#ifdef GP2X
		devgpio = open("/dev/GPIO", O_RDONLY);
		devmem = open("/dev/mem", O_RDWR);
		devfb = open("/dev/fb0", O_RDWR);

		memregs32 = (uint32_t *) mmap(0, 0x10000, PROT_READ|PROT_WRITE, MAP_SHARED, devmem, 0xc0000000);
		memregs16 = (uint16_t *) memregs32;
		screen16 = calloc (SCREEN_W * SCREEN_H, sizeof(uint16_t));
		if (screen16 == NULL)
			fprintf(stderr, "malloc returned null on screen16\n");
		screen16fb = (uint16_t *) mmap(0, 320*240*sizeof(uint16_t), PROT_WRITE, MAP_SHARED, devfb, 0);

		SDL_Init(SDL_INIT_TIMER | SDL_INIT_AUDIO);
		atexit(SDL_Quit);
	#else
		SDL_Init(SDL_INIT_EVERYTHING);
		atexit(SDL_Quit);
		screen=SDL_SetVideoMode(SCREEN_W*SCREEN_SCALE, SCREEN_H*SCREEN_SCALE, 32, SDL_HWSURFACE | SDL_DOUBLEBUF);
		screen16 = calloc (SCREEN_W * SCREEN_H, sizeof(uint16_t));
		if (screen16 == NULL)
			fprintf(stderr, "malloc returned null on screen16\n");

		screen_scaled = calloc (SCREEN_W * SCREEN_SCALE * SCREEN_H * SCREEN_SCALE, sizeof(uint16_t));
		if (screen_scaled == NULL)
			fprintf(stderr, "malloc returned null on screen_scaled\n");
	#endif


	//********Sound Init********

	SDL_AudioSpec sound_format;

	sound_format.freq = SOUND_SAMPLERATE;
	sound_format.format = AUDIO_S16;
	sound_format.channels = 1;
	sound_format.samples = SOUND_BUFFERLEN;		// 512 / 44100 = 11.6 ms
	sound_format.callback = audio_callback;
	sound_format.userdata = NULL;

	if (SDL_OpenAudio(&sound_format, NULL) < 0)
	{
		fprintf(stderr, "Unable to open audio: %s\n", SDL_GetError());
		exit(EXIT_FAILURE);
	}

	//********Sound loading********
	
	bounce_sound.snd = wav_in("sounds/finished/ball_bounce.wav", &bounce_sound.len);
	bounce_sound.pos = bounce_sound.len;
	bounce_sound.vol_mod = (int16_t) (20.0f * 256.0f);		// global volume modifier in 8.8 format

	hit_sound.snd = wav_in("sounds/hit2.bmp.wav", &hit_sound.len);
	hit_sound.pos = hit_sound.len;
	hit_sound.vol_mod = (int16_t) (0.4f * 256.0f);		// global volume modifier in 8.8 format

	global_p = &hit_sound.pos;

	fast_sound.snd = wav_in("sounds/fast2.wav", &fast_sound.len);
	fast_sound.pos = fast_sound.len;
	fast_sound.vol_mod = (int16_t) (1.f * 256.0f);		// global volume modifier in 8.8 format

	//--------Sound loading--------

	SDL_PauseAudio(0);

	//--------Sound Init--------	

	#ifdef GP2X
		uint16_t foo_w, foo_h;
		int16_t iy, ix;
		uint32_t *splash = (uint32_t *) load_png_file("splash.png", &foo_w, &foo_h, &trash1, 5);
		for (iy=0; iy<240; iy++)
			for(ix=0; ix<320; ix++)
				screen16fb[iy * 320 + ix] = splash[iy * 320 + ix] >> 16;
		uint32_t *map = (uint32_t *) load_png_file("map.png", &foo_w, &foo_h, &trash1, 5);
		free(splash);
	#endif

	/********SPRITE LOADING********/
	bg.RGBA = (uint32_t *) load_png_file("background.png", &bg.width, &bg.height, &trash1, 5);
	ball.RGBA = (uint32_t *) load_png_file("ball.png", &ball.width, &ball.height, &trash1, 5);
	ball.alpha = load_png_file("ball_alpha.png", &ball.width, &ball.height, &trash1, 2);
	add_alpha(ball.RGBA, ball.alpha, ball.width, ball.height, trash1, 1);

	// Score board sprites
	
	point_sprite[0] = load_png_file("board/00.png", &point_w, &point_h, &trash1, 2);
	point_sprite[1] = load_png_file("board/15.png", &point_w, &point_h, &trash1, 2);
	point_sprite[2] = load_png_file("board/30.png", &point_w, &point_h, &trash1, 2);
	point_sprite[3] = load_png_file("board/40.png", &point_w, &point_h, &trash1, 2);
	point_sprite[4] = load_png_file("board/a.png", &point_w, &point_h, &trash1, 2);

	for (i=0; i<10; i++)
	{
		sprintf(filename, "board/%d.png", i);
		score_sprite[i] = load_png_file(filename, &score_w, &score_h, &trash1, 2);
	}
	/******************************/

//printf("%d, %d, %d, %d <- ANGLE\n", LUT_LOW_ANG, LUT_HIG_ANG, LUT_AC, LUT_ANG_STEP);

	ballistics_lut();
//printf("%d/%d = %.3f\n", i, LUT_AC*LUT_HC*LUT_DC, (float) i/(LUT_AC*LUT_HC*LUT_DC));
//fprintf(stderr, "average = %.3f\n", (float) somme/(float) foobar);

	ball.x = -30<<FRAC;
	ball.y = 10<<FRAC;
	ball.z = (BOX_D<<FRAC) - BALL_RADIUS - (1<<FRAC);

	/*ball.speed.x = 0;
	ball.speed.y = 0<<FRAC;
	ball.speed.z = 0<<FRAC;*/

//global_p = &box2.x1;

	box1.name = 1;
	box1.x1 = -(BOX_MAX>>1);
	box1.x2 = (BOX_MAX>>1);
	box1.y1 = 0;
	box1.y2 = BOX_MIN;
	box1.z1 = -BOX_D - BOX_THICK<<FRAC;
	box1.z2 = -BOX_D<<FRAC;
	box1.colorA = 0x8460;	//134 143 0
	box1.colorB = 0xB5E1;	//183 189 15
	box1.colorC = 0xEFA2;	//236 247 20
	box1.left = 0;
	box1.right = 0;
	box1.jump = 0;
	box1.moving = 0;
	box1.flying = 0;
	box1.stop = 0;
	box1.ai = 1;
	box1.randir = SIDE_DISTANCE + rand()%(256 - (SIDE_DISTANCE>>1));		// random direction in 0.8 format
	box1.jump_distance=40;
	box1.mov_distance=22;
	box1.phys_jump=0;

	box2.name = 2;
	box2.x1 = -(BOX_MAX>>1);
	box2.x2 = (BOX_MAX>>1);
	box2.y1 = 0;
	box2.y2 = BOX_MIN;
	box2.z1 = BOX_D<<FRAC;
	box2.z2 = BOX_D + BOX_THICK<<FRAC;
	box2.colorA = 0x8460;	//134 143 0
	box2.colorB = 0xB5E1;	//183 189 15
	box2.colorC = 0xEFA2;	//236 247 20
	box2.left = 0;
	box2.right = 0;
	box2.jump = 0;
	box2.moving = 0;
	box2.flying = 0;
	box2.stop = 0;
	box2.ai = 0;
	box2.randir = SIDE_DISTANCE + rand()%(256 - (SIDE_DISTANCE>>1));		// random direction in 0.8 format
	box2.jump_distance=40;
	box2.mov_distance=22;
	box2.phys_jump=0;

	tick = SDL_GetTicks() - 15;

	tgt_x = 0;
	ball_throw(&ball, box2, 32);
	hitting_box = 0;

float circle_size = 2;

	while (exit_key==0)// && i<1)
	{
		//i++;
		#ifndef GP2X
		while (SDL_PollEvent(&event))
		{
			if (event.type==SDL_QUIT)
			{
				exit_key = 1;
				break;
			}

			if (event.type==SDL_KEYDOWN)
				switch (event.key.keysym.sym)
				{
					case SDLK_ESCAPE:
						exit_key=1;
						break;
					case SDLK_r:
						reset_key=1;
						break;
					case SDLK_u:
						leftAIkey=1;
						break;
					case SDLK_i:
						rightAIkey=1;
						break;
					case SDLK_b:
						break_key=1;
						break;
					case SDLK_d:
						d_key=1;
						break;
					/*case SDLK_UP:
						ball_up=1;
						break;
					case SDLK_DOWN:
						ball_down=1;
						break;
					case SDLK_LEFT:
						ball_left=1;
						break;
					case SDLK_RIGHT:
						ball_right=1;
						break;*/
					case SDLK_x:
						box1.left=1;
						break;
					case SDLK_v:
						box1.right=1;
						break;
					case SDLK_c:
						box1.jump=1;
						box1.phys_jump=1;
						break;
					case SDLK_KP1:
						box2.left=1;
						break;
					case SDLK_KP3:
						box2.right=1;
						break;
					case SDLK_KP2:
						box2.jump=1;
						box2.phys_jump=1;
						break;
					/*case SDLK_KP9:
						ball_throwup=1;
						break;*/
					case SDLK_KP8:
						kp8=1;
						break;
					case SDLK_KP4:
						kp4=1;
						break;
					case SDLK_KP5:
						kp5=1;
						break;
					case SDLK_KP6:
						kp6=1;
						break;
				}
			if (event.type==SDL_KEYUP)
				switch (event.key.keysym.sym)
				{
					case SDLK_d:
						d_key=0;
						break;
					case SDLK_UP:
						ball_up=0;
						break;
					case SDLK_DOWN:
						ball_down=0;
						break;
					case SDLK_LEFT:
						ball_left=0;
						break;
					case SDLK_RIGHT:
						ball_right=0;
						break;
					case SDLK_x:
						box1.left=0;
						break;
					case SDLK_v:
						box1.right=0;
						break;
					case SDLK_c:
						box1.jump=0;
						box1.phys_jump=0;
						break;
					case SDLK_KP1:
						box2.left=0;
						break;
					case SDLK_KP3:
						box2.right=0;
						break;
					case SDLK_KP2:
						box2.jump=0;
						box2.phys_jump=0;
						break;
					case SDLK_KP9:
						ball_throwup=0;
						break;
					case SDLK_KP8:
						kp8=0;
						break;
					case SDLK_KP4:
						kp4=0;
						break;
					case SDLK_KP5:
						kp5=0;
						break;
					case SDLK_KP6:
						kp6=0;
						break;
				}
		}
		#endif

		#ifdef GP2X

		/****Input handling****/
		read(devgpio, &io_word, 4);
		io_word &= 0x7FFFF;
		if (io_word != prev_io)
		{
			io_parse(io_word, prev_io);
			prev_io = io_word;

			if (GPB_START == 1)
				exit_key=1;
			if (GPB_A == 1)
				leftAIkey=1;
			if (GPB_Y == 1)
				rightAIkey=1;

			if (GPB_S == 1 || GPB_SW == 1 || GPB_W == 1)
				box1.left=1;
			if (GPB_S == 0 && GPB_SW == 0 && GPB_W == 0)
				box1.left=0;
			if (GPB_N == 1 || GPB_NE == 1 || GPB_E == 1)
				box1.right=1;
			if (GPB_N == 0 && GPB_NE == 0 && GPB_E == 0)
				box1.right=0;
			if (GPB_L == 1)
			{
				box1.jump=1;
				box1.phys_jump=1;
			}
			if (GPB_X == 1)
				box2.left=1;
			if (GPB_B == 1)
				box2.right=1;
			if (GPB_R == 1)
			{
				box2.jump=1;
				box2.phys_jump=1;
			}
			if (GPB_L == 0)
			{
				box1.jump=0;
				box1.phys_jump=0;
			}
			if (GPB_X == 0)
				box2.left=0;
			if (GPB_B == 0)
				box2.right=0;
			if (GPB_R == 0)
			{
				box2.jump=0;
				box2.phys_jump=0;
			}
			if (GPB_VOLM == 1)
				cpu_down=1;
			if (GPB_VOLP == 1)
				cpu_up=1;

			if (GPB_SELECT == 1)
			{
				for (iy=0; iy<240; iy++)
					for(ix=0; ix<320; ix++)
						screen16fb[iy * 320 + ix] = map[iy * 320 + ix] >> 16;

				while (GPB_SELECT == 1)
				{
					read(devgpio, &io_word, 4);
					io_word &= 0x7FFFF;
					if (io_word != prev_io)
					{
						io_parse(io_word, prev_io);
						prev_io = io_word;
					}
					SDL_Delay(15);
				}
				while (GPB_SELECT == 0)
				{
					read(devgpio, &io_word, 4);
					io_word &= 0x7FFFF;
					if (io_word != prev_io)
					{
						io_parse(io_word, prev_io);
						prev_io = io_word;
					}
					SDL_Delay(15);
				}
			}
		}
		/***************/
		#endif

		/*#ifdef GP2X

		if (cpu_down==1)
		{
			cpu_down=0;
			cpu_speed--;
		}
		if (cpu_up==1)
		{
			cpu_up=0;
			cpu_speed++;
		}
		CPUSetting(cpu_speed);
		if (ms==0)
			ms=66000;
		sprintf(line1, "%d MHz %d FPS", cpu_speed, 66000/ms);
		#endif*/

		#ifdef GP2X
		CPUSetting(GP2X_SPEED);
		#endif

		/********************Physics********************/

		/********TIME********/
		tick_prev = tick;
		tick = FASTFWD * SDL_GetTicks()/SLOMO;
		ms = (tick - tick_prev) * 66;	// 65536/1000 = 66
		while (ms < 15 * 66)		// if more than 66 FPS
		{
			SDL_Delay(1);
			tick = FASTFWD * SDL_GetTicks()/SLOMO;
			ms = (tick - tick_prev) * 66;
		}
		// 1 ms is roughly 1/65,536 seconds
		if (ms > 50 * 66)
			ms = 50 * 66;	// if FPS < 20
//ms = 20 * 66;

		// BALL RESET

		if (tick - ball_loss_time > BALL_LOSS_MAX_TIME && ball_loss_time!=0	// if the ball is lost and times out
				|| (ball_loss_time!=0 && ball.z>(-12<<FRAC) && ball.z<0))//reset_key==1)	// or gets too close to the net
		{
			ball_loss_time = 0;
			service = 1;
			tgt_x = -(8<<FRAC) + rand()%(16<<FRAC);	// choose aiming target

			if ((game[0] + game[1]) % 2 == 0)	// right player serves
			{
				ball.x = box2.x1 + box2.x2 >> 1;
				ball.y = 3<<FRAC;
				ball.z = (BOX_D<<FRAC) - BALL_RADIUS - (1<<FRAC);
				ball_throw(&ball, box2, 32);
			}
			else
			{
				ball.x = box1.x1 + box1.x2 >> 1;
				ball.y = 3<<FRAC;
				ball.z = (-BOX_D<<FRAC) + BALL_RADIUS + (1<<FRAC);
				ball_throw(&ball, box1, 32);
				ball.speed.z = -ball.speed.z;
			}
	
			hitting_box = 0;

			reset_key = 0;
		}

		/********BALL********/

		ball.prev.x = ball.x;
		ball.prev.y = ball.y;
		ball.prev.z = ball.z;

		if (ball_up==1)
			ball.speed.z -= 5<<FRAC;
		if (ball_down==1)
			ball.speed.z += 5<<FRAC;
		if (ball_left==1)
			ball.speed.x += 5<<FRAC;
		if (ball_right==1)
			ball.speed.x -= 5<<FRAC;
		if (ball_throwup==1)
			ball.speed.y += (int32_t) ms * (int32_t) (2*GRAV * (float) (1<<FRAC)) >> 16;

		// Gravity
		ball.speed.y += (int32_t) ms * (int32_t) (-GRAV * (float) (1<<FRAC)) >> 16;	// gravity in m/s^2 (97.4 to compensate the 65.536/66 imprecision)

		// Air drag
		ball.speed.y -= sign(ball.speed.y) * (int32_t) ms * ((((ball.speed.y * ball.speed.y)>>16) * (int32_t) ((float) (1 << 10+12) * GRAV / (BALL_TV * BALL_TV))) >> 16) >>  12;
		ball.speed.x -= sign(ball.speed.x) * (int32_t) ms * ((((ball.speed.x * ball.speed.x)>>16) * (int32_t) ((float) (1 << 10+12) * GRAV / (BALL_TV * BALL_TV))) >> 16) >>  12;
		ball.speed.z -= sign(ball.speed.z) * (int32_t) ms * ((((ball.speed.z * ball.speed.z)>>16) * (int32_t) ((float) (1 << 10+12) * GRAV / (BALL_TV * BALL_TV))) >> 16) >>  12;

		// Position calculation
		ball.y += (int32_t) (ball.speed.y * ms) >> 16;
		ball.x += (int32_t) (ball.speed.x * ms) >> 16;
		ball.z += (int32_t) (ball.speed.z * ms) >> 16;

		// Ground bounce
		if (ball.y <= BALL_RADIUS)
		{
			bounce_sound.pos = 0;
			bounce_sound.vol = ball.speed.y * ball.speed.y >> 16;

			ball.y = BALL_RADIUS + BALL_RADIUS-ball.y;
			ball.speed.y = -ball.speed.y * (int32_t) (GROUND_BOUNCE * 256.0f) >> 8;
		}

		/********BOX********/

		if (leftAIkey == 1)
		{
			leftAIkey = 0;
			if (box1.ai == 1)
				box1.ai = 0;
			else
				box1.ai = 1;
		}
		if (rightAIkey == 1)
		{
			rightAIkey = 0;
			if (box2.ai == 1)
				box2.ai = 0;
			else
				box2.ai = 1;
		}

		if (box1.ai==1 && box2.ai==0 && box1.phys_jump==1)	// allows the use of the AI's physical jump button when in AI vs. Human
			box2.jump = 1;
		if (box1.ai==1 && box2.ai==0 && box1.phys_jump==0 && box2.phys_jump==0)
			box2.jump = 0;

		if (box1.ai==0 && box2.ai==1 && box2.phys_jump==1)
			box1.jump = 1;
		if (box1.ai==0 && box2.ai==1 && box1.phys_jump==0 && box2.phys_jump==0)
			box1.jump = 0;

		if (box1.ai==1)
		{
			if (break_key == 1)
				break_key = 0;
			if (prev_hb!=hitting_box || hitting_box == 0)
			{
				box1.randir = SIDE_DISTANCE + rand()%(256 - (SIDE_DISTANCE>>1));
				box1.jump_distance = (JD_MIN + (rand()%JD_RANGE) << FRAC);
				box1.mov_distance = ((MOV_DIST + rand()%MOV_RANGE) << FRAC);
				box1.stop = 0;
				box1.g.x = box1.x1 + box1.x2 >> 1;
				if (box1.g.x > tgt_x)
					box1.left_of_tgt_x = 1;
				else
					box1.left_of_tgt_x = 0;

				if (sign(ball.speed.z) == -1)
					hitting_box = 2;
				if (sign(ball.speed.z) == 1)
					hitting_box = 1;
			}

			box1.g.x = box1.x1 + ((box1.x2-box1.x1) * box1.randir >> 8);	// formerly the average, now the desired random hit spot
			if (hitting_box==1)
			{
				box1.left = 0;
				box1.right = 0;
				box1.jump = 0;
			}
			if (hitting_box==2)
			{
				if (box1.left_of_tgt_x == 1 && box1.g.x > tgt_x && (ball.z - box1.z2 < box1.mov_distance || (ball_jump==1 && ball.z - box1.z2 < box1.jump_distance + box1.mov_distance)))
				{
					box1.left = 0;
					box1.right = 1;
					box1.jump = 0;

					if (box1.moving==1 && box1.randir > 128 && box1.g.x < tgt_x + (BOX_SPD * ms >> 16))	// if the "last" (left one) leg is moving, predict a potential ball miss
							box1.stop = 1;
				}
				else
					if (box1.left_of_tgt_x == 0 && box1.g.x < tgt_x && (ball.z - box1.z2 < box1.mov_distance || (ball_jump==1 && ball.z - box1.z2 < box1.jump_distance + box1.mov_distance)))
					{
						box1.left = 1;
						box1.right = 0;
						box1.jump = 0;

						if (box1.moving==2 && box1.randir < 128 && box1.g.x > tgt_x - (BOX_SPD * ms >> 16))	// if the "last" (right one) leg is moving, predict a potential ball miss
							box1.stop = 1;
					}
					else
					{
						box1.left = 0;
						box1.right = 0;
						box1.jump = 0;
					}
			}
			if (ball_loss_time==0)
			{
				if ((box1.x1 + box1.x2 >> 1) > 0)
					box1.left_of_0 = 1;
				else
					box1.left_of_0 = 0;
			}
			if (tick - ball_loss_time>LOSS_BALL_AI_STOP && ball_loss_time!=0)
			{
				box1.stop = 0;
				if (box1.left_of_0 == 1 && (box1.x1 + box1.x2 >> 1) > 0)
				{
					box1.left = 0;
					box1.right = 1;
					box1.jump = 0;
				}
				else
					if (box1.left_of_0 == 0 && (box1.x1 + box1.x2 >> 1) < 0)
					{
						box1.left = 1;
						box1.right = 0;
						box1.jump = 0;
					}
					else
					{
						box1.left = 0;
						box1.right = 0;
						box1.jump = 0;
					}
			}
			if (ball_jump == 1 && ball_loss_time==0 && ball.z - box1.z2 < box1.jump_distance)	// jumping if the ball's high
			{
				box1.jump = 1;
				if (box1.flying == 2)
				{
					ball_jump = 0;
				}
			}

			if (ball.z + ((int32_t) (ball.speed.z * ms) >> 16) <= -(BOX_D<<FRAC) && ball_loss_time==0 && (rand()%256) < JUMP_ODDS)		// random high ball throwing
			{
				box1.jump = 1;
				if (box1.flying == 2)
				{
					ball_jump = 0;
				}
			}

			if (box1.stop == 1)
			{
				box1.left = 0;
				box1.right = 0;
			}	
		}
/**********************************************************************/
		if (box2.ai==1)
		{
			if (break_key == 1)
				break_key = 0;
			if (prev_hb!=hitting_box || hitting_box == 0)
			{
				box2.randir = SIDE_DISTANCE + rand()%(256 - (SIDE_DISTANCE>>1));
				box2.jump_distance = (JD_MIN + (rand()%JD_RANGE) << FRAC);
				box2.mov_distance = ((MOV_DIST + rand()%MOV_RANGE) << FRAC);
				box2.stop = 0;
				box2.g.x = box2.x1 + box2.x2 >> 1;
				if (box2.g.x > tgt_x)
					box2.left_of_tgt_x = 1;
				else
					box2.left_of_tgt_x = 0;

				if (sign(ball.speed.z) == -1)
					hitting_box = 2;
				if (sign(ball.speed.z) == 1)
					hitting_box = 1;
			}

			box2.g.x = box2.x1 + ((box2.x2-box2.x1) * box2.randir >> 8);	// formerly the average, now the desired random hit spot
			if (hitting_box==2)
			{
				box2.left = 0;
				box2.right = 0;
				box2.jump = 0;
			}
			if (hitting_box==1)
			{
				if (box2.left_of_tgt_x == 1 && box2.g.x > tgt_x && (box2.z1 - ball.z < box2.mov_distance || (ball_jump==1 && box2.z1 - ball.z < box2.jump_distance + box2.mov_distance)))
				{
					box2.left = 0;
					box2.right = 1;
					box2.jump = 0;

					if (box2.moving==1 && box2.randir > 128 && box2.g.x < tgt_x + (BOX_SPD * ms >> 16))	// if the "last" (left one) leg is moving, predict a potential ball miss
							box2.stop = 1;
				}
				else
					if (box2.left_of_tgt_x == 0 && box2.g.x < tgt_x && (box2.z1 - ball.z < box2.mov_distance || (ball_jump==1 && box2.z1 - ball.z < box2.jump_distance + box2.mov_distance)))
					{
						box2.left = 1;
						box2.right = 0;
						box2.jump = 0;

						if (box2.moving==2 && box2.randir < 128 && box2.g.x > tgt_x - (BOX_SPD * ms >> 16))	// if the "last" (right one) leg is moving, predict a potential ball miss
							box2.stop = 1;
					}
					else
					{
						box2.left = 0;
						box2.right = 0;
						box2.jump = 0;
					}
			}
			if (ball_loss_time==0)
			{
				if ((box2.x1 + box2.x2 >> 1) > 0)
					box2.left_of_0 = 1;
				else
					box2.left_of_0 = 0;
			}
			if (tick - ball_loss_time>LOSS_BALL_AI_STOP && ball_loss_time!=0)
			{
				box2.stop = 0;
				if (box2.left_of_0 == 1 && (box2.x1 + box2.x2 >> 1) > 0)
				{
					box2.left = 0;
					box2.right = 1;
					box2.jump = 0;
				}
				else
					if (box2.left_of_0 == 0 && (box2.x1 + box2.x2 >> 1) < 0)
					{
						box2.left = 1;
						box2.right = 0;
						box2.jump = 0;
					}
					else
					{
						box2.left = 0;
						box2.right = 0;
						box2.jump = 0;
					}
			}
			if (ball_jump == 1 && ball_loss_time==0 && box2.z1 - ball.z < box2.jump_distance)	// jumping if the ball's high
			{
				box2.jump = 1;
				if (box2.flying == 2)
				{
					ball_jump = 0;
				}
			}

			if (ball.z + ((int32_t) (ball.speed.z * ms) >> 16) >= (BOX_D<<FRAC) && ball_loss_time==0 && (rand()%256) < JUMP_ODDS)		// random high ball throwing
			{
				box2.jump = 1;
				if (box2.flying == 2)
				{
					ball_jump = 0;
				}
			}

			if (box2.stop == 1)
			{
				box2.left = 0;
				box2.right = 0;
			}	
		}

//fprintf(stderr, "DEBUG : box1.speed.x = %d\n", box1.speed.x);

		prev_hb = hitting_box;
		box_mov(&box1, ms, tick);
		box_mov(&box2, ms, tick);

		/********BOX COLLISIONS********/

		col_detection(box1, &ball);
		col_detection(box2, &ball);

		/********WALLS********/

		// Far wall bounce
		if (ball.z < FAR_WALL_Z<<FRAC)
		{
			ball.z += (FAR_WALL_Z<<FRAC) - ball.z;
			ball.speed.z = -ball.speed.z * (int32_t) (WALL_BOUNCE * 256.0f) >> 8;
			ball.speed.x = ball.speed.x * (int32_t) (WALL_BOUNCE * 256.0f) >> 8;
			ball.speed.y = ball.speed.y * (int32_t) (WALL_BOUNCE * 256.0f) >> 8;
		}

		// Near wall bounce
		if (ball.z > NEAR_WALL_Z<<FRAC)
		{
			ball.z += (NEAR_WALL_Z<<FRAC) - ball.z;
			ball.speed.z = -ball.speed.z * (int32_t) (WALL_BOUNCE * 256.0f) >> 8;
			ball.speed.x = ball.speed.x * (int32_t) (WALL_BOUNCE * 256.0f) >> 8;
			ball.speed.y = ball.speed.y * (int32_t) (WALL_BOUNCE * 256.0f) >> 8;
		}

		/********BALL LOSS********/

		if (abs(ball.z) > BOX_D<<FRAC && ball_loss_time==0)
		{
			ball_loss_time = tick;
			if (sign(ball.z) == -1)
				scoring = 2;
			if (sign(ball.z) == 1)
				scoring = 1;
		}

		/********SCORING********/

		if (scoring!=0)		// game for each player, 0 = 0, 1 = 15, 2 = 30, 3 = 40, 4 = A or 40+
		{
			player_a = scoring-1;
			if (scoring == 1)
				player_b = 1;
			else
				player_b = 0;

			switch (point[player_a])
			{
				case 3:		// 40
					if (point[player_b] < 3)	// 40+ - 30 or less (game won)
					{
						point[player_a] = 0;
						point[player_b] = 0;
						game[player_a]++;
						scoring = 0;
					}
					
					if (point[player_b] == 3)	// A - 40
					{
						point[player_a]++;
						scoring = 0;
					}

					if (point[player_b] == 4)	// A - A (Deuce)
					{
						point[player_a] = 3;
						point[player_b] = 3;
						scoring = 0;
					}
					break;
				case 4:		// Advantage
					point[player_a] = 0;
					point[player_b] = 0;
					game[player_a]++;
					scoring = 0;
					break;
			}

			if (scoring != 0) // if 30 or less
			{
				point[player_a]++;
				scoring = 0;
			}

			if (game[player_a] >= 6 && game[player_a] - game[player_b] >= 2)
			{
				game[player_a] = 0;
				game[player_b] = 0;
				set[player_a]++;
			}	// TODO tie break and such

			printf("%s - %s\t%d - %d\t%d - %d\n", point_codes[point[0]], point_codes[point[1]], game[0], game[1], set[0], set[1]);
		}


		/********************Graphics********************/

		blit_center(screen16, bg.RGBA, bg.width, bg.height, SCREEN_W>>1, SCREEN_H>>1, 0);	//replaces zeroarray
sprintf(line1, "%6d", fast_sound.vol * fast_sound.vol_mod >> 8);
//text_blit(screen16, line1, font, 6, 8, 0, 0);

		/****SCORE BLITTING****/
		score_blit(screen16, point_sprite[point[0]], point_w, point_h, 23, 18);
		score_blit(screen16, point_sprite[point[1]], point_w, point_h, 23, 25);
		score_blit(screen16, score_sprite[game[0]%10], score_w, score_h, 33, 16);
		score_blit(screen16, score_sprite[game[1]%10], score_w, score_h, 33, 23);
		score_blit(screen16, score_sprite[set[0]%10], score_w, score_h, 38, 13);
		score_blit(screen16, score_sprite[set[1]%10], score_w, score_h, 38, 20);
		/**********************/

		box_shadow(screen16, box1);
		box_shadow(screen16, box2);
		ball_shadow(screen16, bg.RGBA, ball);

		//if (coord_convZ(box2.x2, box2.y1, box2.z1)>coord_convZ(ball.x, ball.y, ball.z))	//if box2 is closer than the ball
		/*if (break_key == 1)
		{
			break_key = 0;
			SDL_Delay(1000);
		}*/
		if (box1.x2 >= ball.x && box1.y2 >= ball.y && box1.z2 >= ball.z)
		{
			blit_center(screen16, ball.RGBA, ball.width, ball.height, coord_convX(ball.x, ball.y, ball.z), coord_convY(ball.x, ball.y, ball.z), 1);
			draw_box(screen16, box1);
			draw_box(screen16, box2);
		}
		else
		{
			//if (coord_convZ(box1.x1, box1.y1, box1.z2)>coord_convZ(ball.x, ball.y, ball.z))	//if box1 is closer than the ball
			if (box2.x2 >= ball.x && box2.y2 >= ball.y && box2.z2 >= ball.z)
			{
				draw_box(screen16, box1);
				blit_center(screen16, ball.RGBA, ball.width, ball.height, coord_convX(ball.x, ball.y, ball.z), coord_convY(ball.x, ball.y, ball.z), 1);
				draw_box(screen16, box2);
			}
			else
			{
				draw_box(screen16, box1);
				draw_box(screen16, box2);
				blit_center(screen16, ball.RGBA, ball.width, ball.height, coord_convX(ball.x, ball.y, ball.z), coord_convY(ball.x, ball.y, ball.z), 1);
			}
		}

//draw_circle(screen16, circle_size);
//draw_line(screen16, 1, 38, 121, 73, 0x0000);

	//draw_line(screen16, coord_convX(box1.x2, box1.y1, box1.z2), coord_convY(box1.x2, box1.y1, box1.z2), coord_convX(ball.x, ball.y, ball.z), coord_convY(ball.x, ball.y, ball.z), 0x0000);	//line 1

		/********************Blitting********************/

		#ifdef GP2X
			fb_scale(screen16, screen16fb);			
		#else
			//scale_screen((uint32_t *)screen32, (uint32_t *)screen640, 2);
			scale(screen16, screen_scaled, SCREEN_H, SCREEN_W, SCREEN_SCALE);
			surf_array_o(screen, screen_scaled, SCREEN_W*SCREEN_SCALE, SCREEN_H*SCREEN_SCALE);
			SDL_Flip(screen);
			SDL_Delay(5);
			if (d_key)
				SDL_Delay(100);
		#endif
	}

	//printf("DEBUG BT : %d\n", ballistic_test(440<<FRAC, (int32_t) (0.5f * 65536.0f), 88<<FRAC));

	//printf("sqrt of %.3f = %.3f\n", 4009.3f, (float) sqrtq((int32_t) (4009.3f * (float) (1<<FRAC))) / (float) (1<<FRAC));

	#ifndef GP2X
	SDL_FreeSurface(screen);
	#endif

	SDL_CloseAudio();

	#ifdef GP2X
		chdir("/usr/gp2x");
		execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
		//chdir("/mnt/sd/s5");
	#endif

	//printf("ball coord : %.3f, %.3f, %.3f\n", (float) ball.x/64.0f, (float) ball.y/64.0f, (float) ball.z/64.0f);


	
	return 0;
}
